@@ -37,15 +37,38 @@
   (G_TYPE_INSTANCE_GET_CLASS ((obj), NEMO_TYPE_ACTION, NemoActionClass))
 
 
-#define SELECTION_SINGLE_KEY "S"
-#define SELECTION_MULTIPLE_KEY "M"
-#define SELECTION_ANY_KEY "Any"
-#define SELECTION_NONE_KEY "None"
-#define SELECTION_NOT_NONE_KEY "NotNone"
+#define SELECTION_SINGLE_KEY "s"
+#define SELECTION_MULTIPLE_KEY "m"
+#define SELECTION_ANY_KEY "any"
+#define SELECTION_NONE_KEY "none"
+#define SELECTION_NOT_NONE_KEY "notnone"
 
-#define TOKEN_EXEC_URL_LIST "%U"
+#define TOKEN_EXEC_URI_LIST "%U"
 #define TOKEN_EXEC_FILE_LIST "%F"
-#define TOKEN_LABEL_FILE_NAME "%N"
+#define TOKEN_EXEC_PARENT "%P"
+#define TOKEN_EXEC_FILE_NAME "%f"
+#define TOKEN_EXEC_PARENT_NAME "%p"
+#define TOKEN_EXEC_DEVICE "%D"
+
+#define TOKEN_LABEL_FILE_NAME "%N" // Leave in for compatibility, same as TOKEN_EXEC_FILE_NAME
+
+
+#define ACTION_FILE_GROUP "Nemo Action"
+
+#define KEY_ACTIVE "Active"
+#define KEY_NAME "Name"
+#define KEY_COMMENT "Comment"
+#define KEY_EXEC "Exec"
+#define KEY_ICON_NAME "Icon-Name"
+#define KEY_STOCK_ID "Stock-Id"
+#define KEY_SELECTION "Selection"
+#define KEY_EXTENSIONS "Extensions"
+#define KEY_MIME_TYPES "Mimetypes"
+#define KEY_SEPARATOR "Separator"
+#define KEY_QUOTE_TYPE "Quote"
+#define KEY_DEPENDENCIES "Dependencies"
+#define KEY_CONDITIONS "Conditions"
+#define KEY_WHITESPACE "EscapeSpaces"
 
 typedef struct _NemoAction NemoAction;
 typedef struct _NemoActionClass NemoActionClass;
@@ -58,17 +81,42 @@ typedef enum {
     SELECTION_NONE
 } SelectionType;
 
+typedef enum {
+    QUOTE_TYPE_SINGLE = 0,
+    QUOTE_TYPE_DOUBLE,
+    QUOTE_TYPE_BACKTICK,
+    QUOTE_TYPE_NONE
+} QuoteType;
+
+typedef enum {
+    TOKEN_NONE = 0,
+    TOKEN_PATH_LIST,
+    TOKEN_URI_LIST,
+    TOKEN_FILE_DISPLAY_NAME,
+    TOKEN_PARENT_DISPLAY_NAME,
+    TOKEN_PARENT_PATH,
+    TOKEN_DEVICE
+} TokenType;
+
 struct _NemoAction {
     GtkAction parent;
     gchar *key_file_path;
     SelectionType selection_type;
     gchar **extensions;
-    guint ext_length;
+    gchar **mimetypes;
     gchar *exec;
     gchar *parent_dir;
+    gchar **conditions;
+    gchar *separator;
+    QuoteType quote_type;
     gchar *orig_label;
     gchar *orig_tt;
     gboolean use_parent_dir;
+    gboolean log_output;
+    GList *dbus;
+    gboolean dbus_satisfied;
+    gboolean escape_underscores;
+    gboolean escape_space;
 };
 
 struct _NemoActionClass {
@@ -77,19 +125,25 @@ struct _NemoActionClass {
 
 GType         nemo_action_get_type             (void);
 NemoAction   *nemo_action_new                  (const gchar *name, const gchar *path);
-void          nemo_action_activate             (NemoAction *action, GList *selection);
+void          nemo_action_activate             (NemoAction *action, GList *selection, NemoFile *parent);
 SelectionType nemo_action_get_selection_type   (NemoAction *action);
 gchar       **nemo_action_get_extension_list   (NemoAction *action);
-guint         nemo_action_get_extension_count  (NemoAction *action);
+gchar       **nemo_action_get_mimetypes_list   (NemoAction *action);
 void          nemo_action_set_key_file_path    (NemoAction *action, const gchar *path);
 void          nemo_action_set_exec             (NemoAction *action, const gchar *exec);
 void          nemo_action_set_parent_dir       (NemoAction *action, const gchar *parent_dir);
+void          nemo_action_set_separator        (NemoAction *action, const gchar *separator);
+void          nemo_action_set_conditions       (NemoAction *action, gchar **conditions);
 void          nemo_action_set_orig_label       (NemoAction *action, const gchar *orig_label);
 void          nemo_action_set_orig_tt          (NemoAction *action, const gchar *orig_tt);
-gchar        *nemo_action_get_orig_label       (NemoAction *action);
-gchar        *nemo_action_get_orig_tt          (NemoAction *action);
-void          nemo_action_set_label            (NemoAction *action, NemoFile *file);
-void          nemo_action_set_tt               (NemoAction *action, NemoFile *file);
-void          nemo_action_set_extensions       (NemoAction *action, gchar **ext);
+const gchar  *nemo_action_get_orig_label       (NemoAction *action);
+const gchar  *nemo_action_get_orig_tt          (NemoAction *action);
+gchar       **nemo_action_get_conditions       (NemoAction *action);
+gchar        *nemo_action_get_label            (NemoAction *action, GList *selection, NemoFile *parent);
+gchar        *nemo_action_get_tt               (NemoAction *action, GList *selection, NemoFile *parent);
+void          nemo_action_set_extensions       (NemoAction *action, gchar **extensions);
+void          nemo_action_set_mimetypes        (NemoAction *action, gchar **mimetypes);
+gboolean      nemo_action_get_dbus_satisfied   (NemoAction *action);
+gboolean      nemo_action_get_visibility       (NemoAction *action, GList *selection, NemoFile *parent);
 
 #endif /* NEMO_ACTION_H */
