@@ -42,6 +42,7 @@
 
 enum {
 	CONTENTS_CHANGED,
+    LOCATION_MOUNTED,
 	LAST_SIGNAL
 };
 
@@ -56,7 +57,7 @@ enum {
 #define ELLIPSISED_MENU_ITEM_MIN_CHARS  32
 
 static GParamSpec* properties[NUM_PROPERTIES] = { NULL };
-static guint signals[LAST_SIGNAL];
+static guint signals[LAST_SIGNAL] = { 0 };
 
 struct NemoBookmarkDetails
 {
@@ -65,7 +66,7 @@ struct NemoBookmarkDetails
 	GFile *location;
 	GIcon *icon;
 	NemoFile *file;
-	
+
 	char *scroll_file;
 };
 
@@ -104,7 +105,7 @@ nemo_bookmark_update_icon (NemoBookmark *bookmark)
 					  NEMO_FILE_ATTRIBUTES_FOR_ICON)) {
 		DEBUG ("%s: set new icon", nemo_bookmark_get_name (bookmark));
 
-		new_icon = nemo_file_get_gicon (bookmark->details->file, 0);
+		new_icon = nemo_file_get_emblemed_icon (bookmark->details->file, 0);
 		g_object_set (bookmark,
 			      "icon", new_icon,
 			      NULL);
@@ -126,15 +127,58 @@ bookmark_set_name_from_ready_file (NemoBookmark *self,
 	display_name = nemo_file_get_display_name (self->details->file);
 
 	if (nemo_file_is_home (self->details->file)) {
-		nemo_bookmark_set_name_internal (self, _("Home"));
+		nemo_bookmark_set_custom_name (self, _("Home"));
 	} else if (g_strcmp0 (self->details->name, display_name) != 0) {
-		nemo_bookmark_set_name_internal (self, display_name);
+		nemo_bookmark_set_custom_name (self, display_name);
 		DEBUG ("%s: name changed to %s", nemo_bookmark_get_name (self), display_name);
 	}
 
 	g_free (display_name);
 }
 
+static void
+nemo_bookmark_set_icon_to_default (NemoBookmark *bookmark)
+{
+    GIcon *icon, *emblemed_icon, *folder;
+    GEmblem *emblem;
+    char *uri;
+
+    if (g_file_is_native (bookmark->details->location)) {
+        folder = g_themed_icon_new (NEMO_ICON_FOLDER);
+    } else {
+        uri = g_file_get_uri (bookmark->details->location);
+        if (g_str_has_prefix (uri, EEL_SEARCH_URI)) {
+            folder = g_themed_icon_new (NEMO_ICON_FOLDER_SAVED_SEARCH);
+        } else {
+            folder = g_themed_icon_new (NEMO_ICON_FOLDER_REMOTE);
+        }
+        g_free (uri);
+    }
+
+    if (!nemo_bookmark_uri_get_exists (bookmark)) {
+        DEBUG ("%s: file does not exist, add emblem", nemo_bookmark_get_name (bookmark));
+
+        icon = g_themed_icon_new (GTK_STOCK_DIALOG_WARNING);
+        emblem = g_emblem_new (icon);
+
+        emblemed_icon = g_emblemed_icon_new (folder, emblem);
+
+        g_object_unref (emblem);
+        g_object_unref (icon);
+        g_object_unref (folder);
+
+        folder = emblemed_icon;
+    }
+
+    DEBUG ("%s: setting icon to default", nemo_bookmark_get_name (bookmark));
+
+    g_object_set (bookmark,
+              "icon", folder,
+              NULL);
+
+    g_object_unref (folder);
+}
+
 static void
 bookmark_file_changed_callback (NemoFile *file,
 				NemoBookmark *bookmark)
@@ -177,55 +221,13 @@ bookmark_file_changed_callback (NemoFile *file,
 		 */
 		DEBUG ("%s: trashed", nemo_bookmark_get_name (bookmark));
 		nemo_bookmark_disconnect_file (bookmark);
+        nemo_bookmark_set_icon_to_default (bookmark);
 	} else {
 		nemo_bookmark_update_icon (bookmark);
 		bookmark_set_name_from_ready_file (bookmark, file);
 	}
 }
 
-static void
-nemo_bookmark_set_icon_to_default (NemoBookmark *bookmark)
-{
-	GIcon *icon, *emblemed_icon, *folder;
-	GEmblem *emblem;
-	char *uri;
-
-	if (g_file_is_native (bookmark->details->location)) {
-		folder = g_themed_icon_new (NEMO_ICON_FOLDER);
-	} else {
-		uri = nemo_bookmark_get_uri (bookmark);
-		if (g_str_has_prefix (uri, EEL_SEARCH_URI)) {
-			folder = g_themed_icon_new (NEMO_ICON_FOLDER_SAVED_SEARCH);
-		} else {
-			folder = g_themed_icon_new (NEMO_ICON_FOLDER_REMOTE);
-		}
-		g_free (uri);
-	}
-
-	if (nemo_bookmark_uri_known_not_to_exist (bookmark)) {
-		DEBUG ("%s: file does not exist, add emblem", nemo_bookmark_get_name (bookmark));
-
-		icon = g_themed_icon_new (GTK_STOCK_DIALOG_WARNING);
-		emblem = g_emblem_new (icon);
-
-		emblemed_icon = g_emblemed_icon_new (folder, emblem);
-
-		g_object_unref (emblem);
-		g_object_unref (icon);
-		g_object_unref (folder);
-
-		folder = emblemed_icon;
-	}
-
-	DEBUG ("%s: setting icon to default", nemo_bookmark_get_name (bookmark));
-
-	g_object_set (bookmark,
-		      "icon", folder,
-		      NULL);
-
-	g_object_unref (folder);
-}
-
 static void
 nemo_bookmark_disconnect_file (NemoBookmark *bookmark)
 {
@@ -233,9 +235,10 @@ nemo_bookmark_disconnect_file (NemoBookmark *bookmark)
 		DEBUG ("%s: disconnecting file",
 		       nemo_bookmark_get_name (bookmark));
 
-		g_signal_handlers_disconnect_by_func (bookmark->details->file,
-						      G_CALLBACK (bookmark_file_changed_callback),
-						      bookmark);
+        g_signal_handlers_disconnect_by_func (bookmark->details->file,
+                                              G_CALLBACK (bookmark_file_changed_callback),
+                                              bookmark);
+
 		g_clear_object (&bookmark->details->file);
 	}
 }
@@ -249,14 +252,16 @@ nemo_bookmark_connect_file (NemoBookmark *bookmark)
 		return;
 	}
 
-	if (!nemo_bookmark_uri_known_not_to_exist (bookmark)) {
-		DEBUG ("%s: creating file", nemo_bookmark_get_name (bookmark));
+	if (nemo_bookmark_uri_get_exists (bookmark)) {
+        DEBUG ("%s: creating file", nemo_bookmark_get_name (bookmark));
 
 		bookmark->details->file = nemo_file_get (bookmark->details->location);
-		g_assert (!nemo_file_is_gone (bookmark->details->file));
 
-		g_signal_connect_object (bookmark->details->file, "changed",
-					 G_CALLBACK (bookmark_file_changed_callback), bookmark, 0);
+        g_assert (!nemo_file_is_gone (bookmark->details->file));
+
+        g_signal_connect_object (bookmark->details->file, "changed",
+                                 G_CALLBACK (bookmark_file_changed_callback),
+                                 bookmark, 0);
 	}
 
 	/* Set icon based on available information. */
@@ -359,14 +364,6 @@ nemo_bookmark_finalize (GObject *object)
 	G_OBJECT_CLASS (nemo_bookmark_parent_class)->finalize (object);
 }
 
-static void
-nemo_bookmark_constructed (GObject *obj)
-{
-	NemoBookmark *self = NEMO_BOOKMARK (obj);
-
-	nemo_bookmark_connect_file (self);
-}
-
 static void
 nemo_bookmark_class_init (NemoBookmarkClass *class)
 {
@@ -375,7 +372,6 @@ nemo_bookmark_class_init (NemoBookmarkClass *class)
 	oclass->finalize = nemo_bookmark_finalize;
 	oclass->get_property = nemo_bookmark_get_property;
 	oclass->set_property = nemo_bookmark_set_property;
-	oclass->constructed = nemo_bookmark_constructed;
 
 	signals[CONTENTS_CHANGED] =
 		g_signal_new ("contents-changed",
@@ -386,6 +382,16 @@ nemo_bookmark_class_init (NemoBookmarkClass *class)
 		              g_cclosure_marshal_VOID__VOID,
 		              G_TYPE_NONE, 0);
 
+    signals[LOCATION_MOUNTED] =
+        g_signal_new ("location-mounted",
+                      G_TYPE_FROM_CLASS (class),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (NemoBookmarkClass, location_mounted),
+                      NULL, NULL,
+                      g_cclosure_marshal_generic,
+                      G_TYPE_BOOLEAN, 1,
+                      G_TYPE_FILE);
+
 	properties[PROP_NAME] =
 		g_param_spec_string ("name",
 				     "Bookmark's name",
@@ -585,13 +591,20 @@ nemo_bookmark_new (GFile *location,
                        GIcon *icon)
 {
 	NemoBookmark *new_bookmark;
+    gchar *name;
+
+    if (custom_name == NULL)
+        name = g_file_get_basename (location);
+    else
+        name = g_strdup (custom_name);
 
 	new_bookmark = NEMO_BOOKMARK (g_object_new (NEMO_TYPE_BOOKMARK,
 							"location", location,
 							"icon", icon,
-							"name", custom_name,
+							"name", name,
 							"custom-name", custom_name != NULL,
 							NULL));
+    g_free (name);
 
 	return new_bookmark;
 }				 
@@ -642,22 +655,22 @@ nemo_bookmark_menu_item_new (NemoBookmark *bookmark)
 }
 
 gboolean
-nemo_bookmark_uri_known_not_to_exist (NemoBookmark *bookmark)
+nemo_bookmark_uri_get_exists (NemoBookmark *bookmark)
 {
 	char *path_name;
-	gboolean exists;
+	gboolean exists = FALSE;
 
-	/* Convert to a path, returning FALSE if not local. */
-	if (!g_file_is_native (bookmark->details->location)) {
-		return FALSE;
-	}
-	path_name = g_file_get_path (bookmark->details->location);
+    path_name = g_file_get_path (bookmark->details->location);
+
+	if (g_file_is_native (bookmark->details->location) && g_file_test (path_name, G_FILE_TEST_EXISTS)) {
+		exists = TRUE;
+	} else {
+        g_signal_emit_by_name (bookmark, "location-mounted", bookmark->details->location, &exists);
+    }
 
-	/* Now check if the file exists (sync. call OK because it is local). */
-	exists = g_file_test (path_name, G_FILE_TEST_EXISTS);
 	g_free (path_name);
 
-	return !exists;
+	return exists;
 }
 
 void
@@ -673,3 +686,9 @@ nemo_bookmark_get_scroll_pos (NemoBookmark      *bookmark)
 {
 	return g_strdup (bookmark->details->scroll_file);
 }
+
+void
+nemo_bookmark_connect (NemoBookmark *bookmark)
+{
+    nemo_bookmark_connect_file (bookmark);
+}
\ No newline at end of file
