@@ -272,7 +272,7 @@ nemo_desktop_link_get_date (NemoDesktopLink *link,
 gboolean
 nemo_desktop_link_can_rename (NemoDesktopLink     *link)
 {
-	return (link->details->type == NEMO_DESKTOP_LINK_HOME ||
+	return !(link->details->type == NEMO_DESKTOP_LINK_HOME ||
 		link->details->type == NEMO_DESKTOP_LINK_TRASH ||
 		link->details->type == NEMO_DESKTOP_LINK_NETWORK ||
 		link->details->type == NEMO_DESKTOP_LINK_COMPUTER);
