@@ -52,7 +52,6 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
-#include <libnemo-private/nemo-desktop-background.h>
 #include <libnemo-private/nemo-file-utilities.h>
 #include <libnemo-private/nemo-file-changes-queue.h>
 #include <stdio.h>
@@ -587,8 +586,6 @@ nemo_icon_container_selection_items_local (NemoIconContainer *container,
 	/* must have at least one item */
 	g_assert (items);
 
-	result = FALSE;
-
 	/* get the URI associated with the container */
 	container_uri_string = get_container_uri (container);
 	
@@ -992,55 +989,16 @@ nemo_icon_container_find_drop_target (NemoIconContainer *container,
 	return nemo_icon_container_get_icon_drop_target_uri (container, drop_target_icon);
 }
 
-static gboolean
-selection_is_image_file (GList *selection_list)
-{
-	const char *mime_type;
-	NemoDragSelectionItem *selected_item;
-	gboolean result;
-	GFile *location;
-	GFileInfo *info;
-
-	/* Make sure only one item is selected */
-	if (selection_list == NULL ||
-	    selection_list->next != NULL) {
-		return FALSE;
-	}
-
-	selected_item = selection_list->data;
-
-	mime_type = NULL;
-	
-	location = g_file_new_for_uri (selected_item->uri);
-	info = g_file_query_info (location,
-				  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
-				  0, NULL, NULL);
-	if (info) {
-		mime_type = g_file_info_get_content_type (info);
-	}
-
-	result = eel_istr_has_prefix (mime_type, "image/");
-
-	if (info) {
-		g_object_unref (info);
-	}
-	g_object_unref (location);
-	
-	return result;
-}
-
-
 static void
 nemo_icon_container_receive_dropped_icons (NemoIconContainer *container,
 					       GdkDragContext *context,
 					       int x, int y)
 {
-	char *drop_target, *container_uri;
+	char *drop_target;
 	gboolean local_move_only;
 	double world_x, world_y;
 	gboolean icon_hit;
 	GdkDragAction action, real_action;
-	NemoDragSelectionItem *selected_item;
 
 	drop_target = NULL;
 
