@@ -215,6 +215,7 @@ vfs_file_get_item_count (NemoFile *file,
 	if (count != NULL) {
 		*count = file->details->directory_count;
 	}
+
 	return TRUE;
 }
 
@@ -223,6 +224,7 @@ vfs_file_get_deep_counts (NemoFile *file,
 			  guint *directory_count,
 			  guint *file_count,
 			  guint *unreadable_directory_count,
+              guint *hidden_count,
 			  goffset *total_size)
 {
 	GFileType type;
@@ -239,6 +241,9 @@ vfs_file_get_deep_counts (NemoFile *file,
 	if (total_size != NULL) {
 		*total_size = 0;
 	}
+    if (hidden_count != NULL) {
+        *hidden_count = 0;
+    }
 
 	if (!nemo_file_is_directory (file)) {
 		return NEMO_REQUEST_DONE;
@@ -257,6 +262,9 @@ vfs_file_get_deep_counts (NemoFile *file,
 		if (total_size != NULL) {
 			*total_size = file->details->deep_size;
 		}
+        if (hidden_count != NULL) {
+            *hidden_count = file->details->deep_hidden_count;
+        }
 		return file->details->deep_counts_status;
 	}
 
@@ -337,7 +345,22 @@ vfs_file_get_date (NemoFile *file,
 static char *
 vfs_file_get_where_string (NemoFile *file)
 {
-	return nemo_file_get_parent_uri_for_display (file);
+    GFile *activation_location;
+    NemoFile *location;
+    char *where_string;
+
+    if (!nemo_file_is_in_recent (file)) {
+        location = nemo_file_ref (file);
+    } else {
+        activation_location = nemo_file_get_activation_location (file);
+        location = nemo_file_get (activation_location);
+        g_object_unref (activation_location);
+    }
+
+    where_string = nemo_file_get_parent_uri_for_display (location);
+
+    nemo_file_unref (location);
+    return where_string;
 }
 
 static void
