@@ -31,6 +31,8 @@
 #include <libnemo-private/nemo-file-utilities.h>
 #include <libnemo-private/nemo-file.h>
 #include <libnemo-private/nemo-icon-names.h>
+#define GNOME_DESKTOP_USE_UNSTABLE_API
+#include <libcinnamon-desktop/gnome-desktop-utils.h>
 
 #include <gio/gio.h>
 #include <string.h>
@@ -44,7 +46,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static guint signals[LAST_SIGNAL];
+static guint signals[LAST_SIGNAL] = { 0 };
 static char *window_geometry;
 static NemoBookmarkList *singleton = NULL;
 
@@ -104,12 +106,6 @@ nemo_bookmark_list_get_file (void)
                                  NULL);
     file = g_file_new_for_path (filename);
 
-    if (!g_file_query_exists (file, NULL)) {
-        g_object_unref (file);
-        g_free(filename);
-        return nemo_bookmark_list_get_legacy_file ();
-    }
-
     g_free (filename);
 
     return file;
@@ -137,6 +133,41 @@ bookmark_in_list_notify (GObject *object,
 	g_signal_emit (bookmarks, signals[CHANGED], 0);
 }
 
+static gboolean
+bookmark_location_mounted_callback (NemoBookmark *bookmark,
+                                    GFile *location,
+                                    NemoBookmarkList *bookmarks)
+{
+    gboolean ret = FALSE;
+
+    GList *volumes = g_volume_monitor_get_mounts (bookmarks->volume_monitor);
+    GList *iter = volumes;
+
+    while (iter != NULL) {
+        GMount *mount = G_MOUNT (iter->data);
+        GFile *mount_location = g_mount_get_root (mount);
+
+        gchar *mount_root_uri = g_file_get_uri (mount_location);
+        gchar *location_uri = g_file_get_uri (location);
+
+        ret = g_str_has_prefix (location_uri, mount_root_uri);
+
+        g_free (mount_root_uri);
+        g_free (location_uri);
+
+        g_object_unref (mount_location);
+
+        if (ret == TRUE)
+            break;
+
+        iter = iter->next;
+    }
+
+    g_list_free_full (volumes, (GDestroyNotify) g_object_unref);
+
+    return ret;
+}
+
 static void
 stop_monitoring_bookmark (NemoBookmarkList *bookmarks,
 			  NemoBookmark     *bookmark)
@@ -144,6 +175,9 @@ stop_monitoring_bookmark (NemoBookmarkList *bookmarks,
 	g_signal_handlers_disconnect_by_func (bookmark,
 					      bookmark_in_list_changed_callback,
 					      bookmarks);
+    g_signal_handlers_disconnect_by_func (bookmark,
+                          bookmark_location_mounted_callback,
+                          bookmarks);
 }
 
 static void
@@ -157,7 +191,7 @@ stop_monitoring_one (gpointer data, gpointer user_data)
 }
 
 static void
-clear (NemoBookmarkList *bookmarks)
+clear_bookmarks (NemoBookmarkList *bookmarks)
 {
 	g_list_foreach (bookmarks->list, stop_monitoring_one, bookmarks);
 	g_list_free_full (bookmarks->list, g_object_unref);
@@ -174,7 +208,9 @@ do_finalize (GObject *object)
 
 	g_queue_free (NEMO_BOOKMARK_LIST (object)->pending_ops);
 
-	clear (NEMO_BOOKMARK_LIST (object));
+	clear_bookmarks (NEMO_BOOKMARK_LIST (object));
+
+    g_object_unref (NEMO_BOOKMARK_LIST (object)->volume_monitor);
 
 	G_OBJECT_CLASS (nemo_bookmark_list_parent_class)->finalize (object);
 }
@@ -233,6 +269,14 @@ bookmark_monitor_changed_cb (GFileMonitor      *monitor,
 	}
 }
 
+static void
+volume_monitor_activity_cb (GVolumeMonitor *monitor, GMount *mount, gpointer user_data)
+{
+    NemoBookmarkList *bookmarks = NEMO_BOOKMARK_LIST (user_data);
+
+    nemo_bookmark_list_load_file (bookmarks);
+}
+
 static void
 nemo_bookmark_list_init (NemoBookmarkList *bookmarks)
 {
@@ -250,6 +294,15 @@ nemo_bookmark_list_init (NemoBookmarkList *bookmarks)
 			  G_CALLBACK (bookmark_monitor_changed_cb), bookmarks);
 
 	g_object_unref (file);
+
+    bookmarks->volume_monitor = g_volume_monitor_get ();
+
+    g_signal_connect (bookmarks->volume_monitor, "mount-added",
+                      G_CALLBACK (volume_monitor_activity_cb), bookmarks);
+    g_signal_connect (bookmarks->volume_monitor, "mount-removed",
+                      G_CALLBACK (volume_monitor_activity_cb), bookmarks);
+    g_signal_connect (bookmarks->volume_monitor, "mount-changed",
+                      G_CALLBACK (volume_monitor_activity_cb), bookmarks);
 }
 
 static void
@@ -259,12 +312,16 @@ insert_bookmark_internal (NemoBookmarkList *bookmarks,
 {
 	bookmarks->list = g_list_insert (bookmarks->list, bookmark, index);
 
+    g_signal_connect_object (bookmark, "location-mounted",
+                 G_CALLBACK (bookmark_location_mounted_callback), bookmarks, 0);
 	g_signal_connect_object (bookmark, "contents-changed",
 				 G_CALLBACK (bookmark_in_list_changed_callback), bookmarks, 0);
 	g_signal_connect_object (bookmark, "notify::icon",
 				 G_CALLBACK (bookmark_in_list_notify), bookmarks, 0);
 	g_signal_connect_object (bookmark, "notify::name",
 				 G_CALLBACK (bookmark_in_list_notify), bookmarks, 0);
+
+    nemo_bookmark_connect (bookmark);
 }
 
 /**
@@ -327,6 +384,8 @@ nemo_bookmark_list_delete_item_at (NemoBookmarkList *bookmarks,
 	g_return_if_fail (index < g_list_length (bookmarks->list));
 
 	doomed = g_list_nth (bookmarks->list, index);
+	g_return_if_fail (doomed != NULL);
+
 	bookmarks->list = g_list_remove_link (bookmarks->list, doomed);
 
 	g_assert (NEMO_IS_BOOKMARK (doomed->data));
@@ -357,6 +416,8 @@ nemo_bookmark_list_move_item (NemoBookmarkList *bookmarks,
 	}
 
 	bookmark_item = g_list_nth (bookmarks->list, index);
+	g_return_if_fail (bookmark_item != NULL);
+
 	bookmarks->list = g_list_remove_link (bookmarks->list,
 					      bookmark_item);
 
@@ -373,6 +434,36 @@ nemo_bookmark_list_move_item (NemoBookmarkList *bookmarks,
 	nemo_bookmark_list_save_file (bookmarks);
 }
 
+static gint
+nemo_bookmark_list_compare_func (gconstpointer a, gconstpointer b)
+{
+    g_assert (NEMO_IS_BOOKMARK (a));
+    g_assert (NEMO_IS_BOOKMARK (b));
+
+    return g_utf8_collate (nemo_bookmark_get_name (NEMO_BOOKMARK (a)),
+                           nemo_bookmark_get_name (NEMO_BOOKMARK (b)));
+}
+
+/**
+ * nemo_bookmark_list_sort_ascending:
+ *
+ * Sort bookmarks in ascending order.
+ * @bookmarks: the list of bookmarks.
+ **/
+void
+nemo_bookmark_list_sort_ascending (NemoBookmarkList *bookmarks)
+{
+    g_assert (NEMO_IS_BOOKMARK_LIST (bookmarks));
+
+    bookmarks->list = g_list_sort (
+        bookmarks->list, (GCompareFunc)nemo_bookmark_list_compare_func);
+
+    /* Save bookmarks to file. This will also inform widgets about the changes
+     * we just made to the list.
+     */
+    nemo_bookmark_list_save_file (bookmarks);
+}
+
 /**
  * nemo_bookmark_list_delete_items_with_uri:
  * 
@@ -450,6 +541,28 @@ nemo_bookmark_list_insert_item (NemoBookmarkList *bookmarks,
 	nemo_bookmark_list_save_file (bookmarks);
 }
 
+GList *
+nemo_bookmark_list_get_for_uri (NemoBookmarkList   *bookmarks,
+                                      const char   *uri)
+{
+    g_return_val_if_fail (NEMO_IS_BOOKMARK_LIST (bookmarks), NULL);
+
+    GList *iter;
+    GList *results = NULL;
+    NemoBookmark *bookmark;
+
+    for (iter = bookmarks->list; iter != NULL; iter = iter->next) {
+        bookmark = iter->data;
+        gchar *bm_uri = nemo_bookmark_get_uri (bookmark);
+        if (g_strcmp0 (uri, bm_uri) == 0) {
+            results = g_list_append (results, bookmark);
+        }
+        g_free (bm_uri);
+    }
+
+    return results;
+}
+
 /**
  * nemo_bookmark_list_item_at:
  * 
@@ -537,9 +650,13 @@ load_file_async (NemoBookmarkList *self,
 	GFile *file;
 
 	file = nemo_bookmark_list_get_file ();
+	if (!g_file_query_exists (file, NULL)) {
+		g_object_unref (file);
+		file = nemo_bookmark_list_get_legacy_file ();
+	}
 
 	/* Wipe out old list. */
-	clear (self);
+	clear_bookmarks (self);
 
 	/* keep the bookmark list alive */
 	g_object_ref (self);
@@ -565,6 +682,19 @@ save_file_finish (NemoBookmarkList *bookmarks,
 		g_error_free (error);
 	}
 
+    if (geteuid () == 0) {
+        struct passwd *pwent;
+        pwent = gnome_desktop_get_session_user_pwent ();
+
+        gchar *bookmarks_path = g_file_get_path (G_FILE (source));
+
+        if (g_strcmp0 (pwent->pw_dir, g_get_home_dir ()) == 0) {
+            G_GNUC_UNUSED int res;
+
+            res = chown (bookmarks_path, pwent->pw_uid, pwent->pw_gid);
+        }
+    }
+
 	file = nemo_bookmark_list_get_file ();
 
 	/* re-enable bookmark file monitoring */
@@ -583,8 +713,8 @@ save_file_async (NemoBookmarkList *bookmarks,
 	GFile *file;
 	GList *l;
 	GString *bookmark_string;
-    GFile *parent;
-    char *path;
+	GFile *parent;
+	char *path;
 
 	/* temporarily disable bookmark file monitoring when writing file */
 	if (bookmarks->monitor != NULL) {
@@ -593,6 +723,9 @@ save_file_async (NemoBookmarkList *bookmarks,
 	}
 
 	file = nemo_bookmark_list_get_file ();
+	/* FIXME: `bookmark_string` is currently leaked as `g_string_free()` is
+	 * 	  never called.
+	 */
 	bookmark_string = g_string_new (NULL);
 
 	for (l = bookmarks->list; l; l = l->next) {
@@ -620,11 +753,11 @@ save_file_async (NemoBookmarkList *bookmarks,
 	/* keep the bookmark list alive */
 	g_object_ref (bookmarks);
 
-    parent = g_file_get_parent (file);
-    path = g_file_get_path (parent);
-    g_mkdir_with_parents (path, 0700);
-    g_free (path);
-    g_object_unref (parent);
+	parent = g_file_get_parent (file);
+	path = g_file_get_path (parent);
+	g_mkdir_with_parents (path, 0700);
+	g_free (path);
+	g_object_unref (parent);
 
 	g_file_replace_contents_async (file, bookmark_string->str,
 				       bookmark_string->len, NULL,
@@ -709,21 +842,23 @@ nemo_bookmark_list_save_file (NemoBookmarkList *bookmarks)
 	}
 }
 
+static NemoBookmarkList *list = NULL;
+
 /**
- * nemo_bookmark_list_new:
+ * nemo_bookmark_list_get_default:
  * 
- * Create a new bookmark_list, with contents read from disk.
+ * Retrieves the bookmark list singleton, with contents read from disk.
  * 
- * Return value: A pointer to the new widget.
+ * Return value: A pointer to the object
  **/
 NemoBookmarkList *
-nemo_bookmark_list_new (void)
+nemo_bookmark_list_get_default (void)
 {
-	NemoBookmarkList *list;
+    if (list == NULL) {
+        list = NEMO_BOOKMARK_LIST (g_object_new (NEMO_TYPE_BOOKMARK_LIST, NULL));
+    }
 
-	list = NEMO_BOOKMARK_LIST (g_object_new (NEMO_TYPE_BOOKMARK_LIST, NULL));
-
-	return list;
+    return list;
 }
 
 /**
