@@ -107,7 +107,6 @@ struct NemoIconViewDetails
 	guint icon_merge_id;
 	
 	gboolean filter_by_screen;
-	int num_screens;
 
 	gboolean compact;
 
@@ -178,6 +177,7 @@ static void                 nemo_icon_view_set_zoom_level               (NemoIco
 									     NemoZoomLevel     new_level,
 									     gboolean              always_emit);
 static void                 nemo_icon_view_update_click_mode            (NemoIconView           *icon_view);
+static void                 nemo_icon_view_update_click_to_rename_mode  (NemoIconView           *icon_view);
 static void                 nemo_icon_view_set_directory_tighter_layout (NemoIconView           *icon_view,
                                         NemoFile         *file,
                                         gboolean              tighter_layout);
@@ -481,29 +481,13 @@ nemo_icon_view_clear (NemoView *view)
 static gboolean
 should_show_file_on_screen (NemoView *view, NemoFile *file)
 {
-	char *screen_string;
-	int screen_num;
 	NemoIconView *icon_view;
-	GdkScreen *screen;
 
 	icon_view = NEMO_ICON_VIEW (view);
 
 	if (!nemo_view_should_show_file (view, file)) {
 		return FALSE;
-	}
-	
-	/* Get the screen for this icon from the metadata. */
-	screen_string = nemo_file_get_metadata
-		(file, NEMO_METADATA_KEY_SCREEN, "0");
-	screen_num = atoi (screen_string);
-	g_free (screen_string);
-	screen = gtk_widget_get_screen (GTK_WIDGET (view));
-
-	if (screen_num != gdk_screen_get_number (screen) &&
-	    (screen_num < icon_view->details->num_screens ||
-	     gdk_screen_get_number (screen) > 0)) {
-		return FALSE;
-	}
+	}	
 
 	return TRUE;
 }
@@ -558,6 +542,11 @@ nemo_icon_view_add_file (NemoView *view, NemoFile *file, NemoDirectory *director
 		return;
 	}
 
+    if (nemo_file_has_thumbnail_access_problem (file)) {
+        nemo_application_set_cache_flag (nemo_application_get_singleton ());
+        nemo_window_slot_check_bad_cache_bar (nemo_view_get_nemo_window_slot (view));
+    }
+
 	/* Reset scroll region for the first icon added when loading a directory. */
 	if (nemo_view_get_loading (view) && nemo_icon_container_is_empty (icon_container)) {
 		nemo_icon_container_reset_scroll_region (icon_container);
@@ -1223,7 +1212,7 @@ nemo_icon_view_restore_default_zoom_level (NemoView *view)
 static NemoZoomLevel
 nemo_icon_view_get_default_zoom_level (NemoView *view)
 {
-    g_return_if_fail (NEMO_IS_ICON_VIEW (view));
+    g_return_val_if_fail (NEMO_IS_ICON_VIEW (view), NEMO_ZOOM_LEVEL_NULL);
 
     return get_default_zoom_level(NEMO_ICON_VIEW (view));
 }
@@ -1860,7 +1849,6 @@ nemo_icon_view_filter_by_screen (NemoIconView *icon_view,
 				     gboolean filter)
 {
 	icon_view->details->filter_by_screen = filter;
-	icon_view->details->num_screens = gdk_display_get_n_screens (gtk_widget_get_display (GTK_WIDGET (icon_view)));
 }
 
 static void
@@ -2110,9 +2098,17 @@ get_icon_drop_target_uri_callback (NemoIconContainer *container,
 static void
 nemo_icon_view_click_policy_changed (NemoView *directory_view)
 {
-	g_assert (NEMO_IS_ICON_VIEW (directory_view));
+    g_assert (NEMO_IS_ICON_VIEW (directory_view));
 
-	nemo_icon_view_update_click_mode (NEMO_ICON_VIEW (directory_view));
+    nemo_icon_view_update_click_mode (NEMO_ICON_VIEW (directory_view));
+}
+
+static void
+nemo_icon_view_click_to_rename_mode_changed (NemoView *directory_view)
+{
+    g_assert (NEMO_IS_ICON_VIEW (directory_view));
+
+    nemo_icon_view_update_click_to_rename_mode (NEMO_ICON_VIEW (directory_view));
 }
 
 static void
@@ -2315,6 +2311,22 @@ nemo_icon_view_update_click_mode (NemoIconView *icon_view)
 						       click_mode == NEMO_CLICK_POLICY_SINGLE);
 }
 
+static void
+nemo_icon_view_update_click_to_rename_mode (NemoIconView *icon_view)
+{
+    NemoIconContainer   *icon_container;
+    gboolean enabled;
+
+    icon_container = get_icon_container (icon_view);
+    g_assert (icon_container != NULL);
+
+    enabled = g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_CLICK_TO_RENAME);
+
+    nemo_icon_container_set_click_to_rename_enabled (icon_container,
+                                                     enabled);
+}
+
+
 static gboolean
 get_stored_layout_timestamp (NemoIconContainer *container,
 			     NemoIconData *icon_data,
@@ -2468,6 +2480,7 @@ create_icon_container (NemoIconView *icon_view)
 			   GTK_WIDGET (icon_container));
 
 	nemo_icon_view_update_click_mode (icon_view);
+    nemo_icon_view_update_click_to_rename_mode (icon_view);
 
 	gtk_widget_show (GTK_WIDGET (icon_container));
 
@@ -2684,6 +2697,7 @@ nemo_icon_view_class_init (NemoIconViewClass *klass)
 	nemo_view_class->zoom_to_level = nemo_icon_view_zoom_to_level;
 	nemo_view_class->get_zoom_level = nemo_icon_view_get_zoom_level;
         nemo_view_class->click_policy_changed = nemo_icon_view_click_policy_changed;
+        nemo_view_class->click_to_rename_mode_changed = nemo_icon_view_click_to_rename_mode_changed;
         nemo_view_class->merge_menus = nemo_icon_view_merge_menus;
         nemo_view_class->unmerge_menus = nemo_icon_view_unmerge_menus;
         nemo_view_class->sort_directories_first_changed = nemo_icon_view_sort_directories_first_changed;
