@@ -32,13 +32,16 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <cairo-gobject.h>
 
 #include <libegg/eggtreemultidnd.h>
 #include <eel/eel-graphic-effects.h>
 #include <libnemo-private/nemo-dnd.h>
+#include <libnemo-private/nemo-file-utilities.h>
 
 enum {
 	SUBDIRECTORY_UNLOADED,
+    GET_ICON_SCALE,
 	LAST_SIGNAL
 };
 
@@ -77,6 +80,7 @@ struct NemoListModelDetails {
 	GPtrArray *columns;
 
 	GList *highlight_files;
+    gboolean temp_unsorted;
 };
 
 typedef struct {
@@ -156,7 +160,7 @@ nemo_list_model_get_column_type (GtkTreeModel *tree_model, int index)
 	case NEMO_LIST_MODEL_LARGE_ICON_COLUMN:
 	case NEMO_LIST_MODEL_LARGER_ICON_COLUMN:
 	case NEMO_LIST_MODEL_LARGEST_ICON_COLUMN:
-		return GDK_TYPE_PIXBUF;
+		return CAIRO_GOBJECT_TYPE_SURFACE;
 	case NEMO_LIST_MODEL_FILE_NAME_IS_EDITABLE_COLUMN:
 		return G_TYPE_BOOLEAN;
 	default:
@@ -241,6 +245,21 @@ nemo_list_model_get_path (GtkTreeModel *tree_model, GtkTreeIter *iter)
 	return path;
 }
 
+static gint
+nemo_list_model_get_icon_scale (NemoListModel *model)
+{
+   gint retval = -1;
+
+   g_signal_emit (model, list_model_signals[GET_ICON_SCALE], 0,
+              &retval);
+
+   if (retval == -1) {
+       retval = gdk_screen_get_monitor_scale_factor (gdk_screen_get_default (), 0);
+   }
+
+   return retval;
+}
+
 static void
 nemo_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column, GValue *value)
 {
@@ -253,12 +272,13 @@ nemo_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int colu
 	NemoIconInfo *icon_info;
 	GEmblem *emblem;
 	GList *emblem_icons, *l;
-	int icon_size;
+	int icon_size, icon_scale;
 	NemoZoomLevel zoom_level;
 	NemoFile *parent_file;
 	char *emblems_to_ignore[3];
 	int i;
 	NemoFileIconFlags flags;
+    cairo_surface_t *surface;
 	
 	model = (NemoListModel *)tree_model;
 
@@ -286,11 +306,12 @@ nemo_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int colu
 	case NEMO_LIST_MODEL_LARGE_ICON_COLUMN:
 	case NEMO_LIST_MODEL_LARGER_ICON_COLUMN:
 	case NEMO_LIST_MODEL_LARGEST_ICON_COLUMN:
-		g_value_init (value, GDK_TYPE_PIXBUF);
+		g_value_init (value, CAIRO_GOBJECT_TYPE_SURFACE);
 
 		if (file != NULL) {
 			zoom_level = nemo_list_model_get_zoom_level_from_column_id (column);
-			icon_size = nemo_get_icon_size_for_zoom_level (zoom_level);
+			icon_size = nemo_get_list_icon_size_for_zoom_level (zoom_level);
+            icon_scale = nemo_list_model_get_icon_scale (model);
 
 			flags = NEMO_FILE_ICON_FLAGS_USE_THUMBNAILS |
 				NEMO_FILE_ICON_FLAGS_FORCE_THUMBNAIL_SIZE |
@@ -313,7 +334,7 @@ nemo_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int colu
 				}
 			}
 
-            GdkPixbuf *pixbuf = nemo_file_get_icon_pixbuf (file, icon_size, TRUE, flags);
+            GdkPixbuf *pixbuf = nemo_file_get_icon_pixbuf (file, icon_size, TRUE, icon_scale, flags);
 
             gint w, h, s;
             gboolean bad_ratio;
@@ -324,8 +345,8 @@ nemo_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int colu
             if (s < icon_size)
                 icon_size = s;
 
-            bad_ratio = nemo_icon_get_emblem_size_for_icon_size (icon_size) > w ||
-                        nemo_icon_get_emblem_size_for_icon_size (icon_size) > h;
+            bad_ratio = nemo_icon_get_emblem_size_for_icon_size (icon_size) * icon_scale > w ||
+                        nemo_icon_get_emblem_size_for_icon_size (icon_size) * icon_scale > h;
 
 			gicon = G_ICON (pixbuf);
 
@@ -361,7 +382,7 @@ nemo_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int colu
 
 			g_list_free_full (emblem_icons, g_object_unref);
 
-			icon_info = nemo_icon_info_lookup (gicon, icon_size);
+			icon_info = nemo_icon_info_lookup (gicon, icon_size, icon_scale);
 			icon = nemo_icon_info_get_pixbuf_at_size (icon_info, icon_size);
 
 			g_object_unref (icon_info);
@@ -379,7 +400,8 @@ nemo_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int colu
 				}
 			}
 
-			g_value_set_object (value, icon);
+            surface = gdk_cairo_surface_create_from_pixbuf (icon, icon_scale, NULL);
+            g_value_take_boxed (value, surface);
 			g_object_unref (icon);
 		}
 		break;
@@ -993,9 +1015,11 @@ nemo_list_model_add_file (NemoListModel *model, NemoFile *file,
 		}
 	}
 
-	
-	file_entry->ptr = g_sequence_insert_sorted (files, file_entry,
-					    nemo_list_model_file_entry_compare_func, model);
+	if (model->details->temp_unsorted)
+        file_entry->ptr = g_sequence_append (files, file_entry);
+    else
+        file_entry->ptr = g_sequence_insert_sorted (files, file_entry,
+                                                    nemo_list_model_file_entry_compare_func, model);
 
 	g_hash_table_insert (parent_hash, file, file_entry->ptr);
 	
@@ -1009,16 +1033,28 @@ nemo_list_model_add_file (NemoListModel *model, NemoFile *file,
 		gtk_tree_model_row_inserted (GTK_TREE_MODEL (model), path, &iter);
 	}
 
-	if (nemo_file_is_directory (file)) {
-		file_entry->files = g_sequence_new ((GDestroyNotify)file_entry_free);
+    gboolean add_child = FALSE;
 
-		add_dummy_row (model, file_entry);
+    if (nemo_file_is_directory (file)) {
+        gint count;
+        if (nemo_file_get_directory_item_count (file, &count, NULL)) {
+            add_child = count > 0;
+        } else {
+            add_child = TRUE;
+        }
+    }
+
+    if (add_child) {
+        file_entry->files = g_sequence_new ((GDestroyNotify)file_entry_free);
+
+        add_dummy_row (model, file_entry);
+
+        gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (model),
+                                                      path, &iter);
+    }
 
-		gtk_tree_model_row_has_child_toggled (GTK_TREE_MODEL (model),
-						      path, &iter);
-	}
 	gtk_tree_path_free (path);
-	
+
 	return TRUE;
 }
 
@@ -1040,10 +1076,10 @@ nemo_list_model_file_changed (NemoListModel *model, NemoFile *file,
 		return;
 	}
 
-	
 	pos_before = g_sequence_iter_get_position (ptr);
-		
-	g_sequence_sort_changed (ptr, nemo_list_model_file_entry_compare_func, model);
+
+    if (!model->details->temp_unsorted)
+        g_sequence_sort_changed (ptr, nemo_list_model_file_entry_compare_func, model);
 
 	pos_after = g_sequence_iter_get_position (ptr);
 
@@ -1371,7 +1407,7 @@ nemo_list_model_get_attribute_from_sort_column_id (NemoListModel *model,
 	NemoColumn *column;
 	int index;
 	GQuark attribute;
-	
+
 	index = sort_column_id - NEMO_LIST_MODEL_NUM_COLUMNS;
 
 	if (index < 0 || index >= model->details->columns->len) {
@@ -1426,9 +1462,9 @@ nemo_list_model_get_column_id_from_zoom_level (NemoZoomLevel zoom_level)
 		return NEMO_LIST_MODEL_LARGER_ICON_COLUMN;
 	case NEMO_ZOOM_LEVEL_LARGEST:
 		return NEMO_LIST_MODEL_LARGEST_ICON_COLUMN;
+    default: 
+        g_return_val_if_reached (NEMO_LIST_MODEL_STANDARD_ICON_COLUMN);
 	}
-
-	g_return_val_if_reached (NEMO_LIST_MODEL_STANDARD_ICON_COLUMN);
 }
 
 void
@@ -1575,6 +1611,14 @@ nemo_list_model_class_init (NemoListModelClass *klass)
                       g_cclosure_marshal_VOID__OBJECT,
                       G_TYPE_NONE, 1,
                       NEMO_TYPE_DIRECTORY);
+
+      list_model_signals[GET_ICON_SCALE] =
+        g_signal_new ("get-icon-scale",
+                      NEMO_TYPE_LIST_MODEL,
+                      G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+                      0, NULL, NULL,
+                      NULL,
+                      G_TYPE_INT, 0);
 }
 
 static void
@@ -1693,3 +1737,18 @@ nemo_list_model_set_highlight_for_files (NemoListModel *model,
 
 	}
 }
+
+void
+nemo_list_model_set_temporarily_disable_sort (NemoListModel *model, gboolean disable)
+{
+    model->details->temp_unsorted = disable;
+
+    if (!disable)
+        nemo_list_model_sort (model);
+}
+
+gboolean
+nemo_list_model_get_temporarily_disable_sort (NemoListModel *model)
+{
+    return model->details->temp_unsorted;
+}
