@@ -23,7 +23,7 @@
 
 #include <config.h>
 #include "nemo-query-editor.h"
-#include "nemo-window-slot.h"
+#include "nemo-file-utilities.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
@@ -34,7 +34,6 @@
 #include <gtk/gtk.h>
 
 typedef enum {
-	NEMO_QUERY_EDITOR_ROW_LOCATION,
 	NEMO_QUERY_EDITOR_ROW_TYPE,
 	
 	NEMO_QUERY_EDITOR_ROW_LAST
@@ -67,14 +66,15 @@ struct NemoQueryEditorDetails {
 	gboolean change_frozen;
 	guint typing_timeout_id;
 	gboolean is_visible;
-	GtkWidget *invisible_vbox;
-	GtkWidget *visible_vbox;
+	GtkWidget *vbox;
+    GtkWidget *search_bar_revealer;
+
+	GtkWidget *search_current_button;
+	GtkWidget *search_all_button;
+	char *current_uri;
 
 	GList *rows;
 	char *last_set_query_text;
-	
-	NemoSearchBar *bar;
-	NemoWindowSlot *slot;
 };
 
 enum {
@@ -83,7 +83,7 @@ enum {
 	LAST_SIGNAL
 }; 
 
-static guint signals[LAST_SIGNAL];
+static guint signals[LAST_SIGNAL] = { 0 };
 
 static void entry_activate_cb (GtkWidget *entry, NemoQueryEditor *editor);
 static void entry_changed_cb  (GtkWidget *entry, NemoQueryEditor *editor);
@@ -93,12 +93,6 @@ static void nemo_query_editor_changed (NemoQueryEditor *editor);
 static NemoQueryEditorRow * nemo_query_editor_add_row (NemoQueryEditor *editor,
 							       NemoQueryEditorRowType type);
 
-static GtkWidget *location_row_create_widgets  (NemoQueryEditorRow *row);
-static void       location_row_add_to_query    (NemoQueryEditorRow *row,
-					        NemoQuery          *query);
-static void       location_row_free_data       (NemoQueryEditorRow *row);
-static void       location_add_rows_from_query (NemoQueryEditor    *editor,
-					        NemoQuery          *query);
 static GtkWidget *type_row_create_widgets      (NemoQueryEditorRow *row);
 static void       type_row_add_to_query        (NemoQueryEditorRow *row,
 					        NemoQuery          *query);
@@ -109,12 +103,6 @@ static void       type_add_rows_from_query     (NemoQueryEditor    *editor,
 
 
 static NemoQueryEditorRowOps row_type[] = {
-	{ N_("Location"),
-	  location_row_create_widgets,
-	  location_row_add_to_query,
-	  location_row_free_data,
-	  location_add_rows_from_query
-	},
 	{ N_("File Type"),
 	  type_row_create_widgets,
 	  type_row_add_to_query,
@@ -132,23 +120,10 @@ nemo_query_editor_dispose (GObject *object)
 
 	editor = NEMO_QUERY_EDITOR (object);
 
-	if (editor->details->typing_timeout_id) {
+	if (editor->details->typing_timeout_id > 0) {
 		g_source_remove (editor->details->typing_timeout_id);
 		editor->details->typing_timeout_id = 0;
 	}
-
-	if (editor->details->bar != NULL) {
-		g_signal_handlers_disconnect_by_func (editor->details->entry,
-						      entry_activate_cb,
-						      editor);
-		g_signal_handlers_disconnect_by_func (editor->details->entry,
-						      entry_changed_cb,
-						      editor);
-		
-		nemo_search_bar_return_entry (editor->details->bar);
-		eel_remove_weak_pointer (&editor->details->bar);
-	}
-
 	G_OBJECT_CLASS (nemo_query_editor_parent_class)->dispose (object);
 }
 
@@ -160,9 +135,6 @@ nemo_query_editor_draw (GtkWidget *widget,
 
 	context = gtk_widget_get_style_context (widget);
 
-	gtk_style_context_save (context);
-	gtk_style_context_add_class (context, GTK_STYLE_CLASS_INFO);
-
 	gtk_render_background (context, cr, 0, 0,
 			       gtk_widget_get_allocated_width (widget),
 			       gtk_widget_get_allocated_height (widget));
@@ -171,8 +143,6 @@ nemo_query_editor_draw (GtkWidget *widget,
 			  gtk_widget_get_allocated_width (widget),
 			  gtk_widget_get_allocated_height (widget));
 
-	gtk_style_context_restore (context);
-
 	GTK_WIDGET_CLASS (nemo_query_editor_parent_class)->draw (widget, cr);
 
 	return FALSE;
@@ -215,10 +185,19 @@ nemo_query_editor_class_init (NemoQueryEditorClass *class)
 	g_type_class_add_private (class, sizeof (NemoQueryEditorDetails));
 }
 
+GFile *
+nemo_query_editor_get_location (NemoQueryEditor *editor)
+{
+	GFile *file = NULL;
+	if (editor->details->current_uri != NULL)
+		file = g_file_new_for_uri (editor->details->current_uri);
+	return file;
+}
+
 static void
 entry_activate_cb (GtkWidget *entry, NemoQueryEditor *editor)
 {
-	if (editor->details->typing_timeout_id) {
+	if (editor->details->typing_timeout_id > 0) {
 		g_source_remove (editor->details->typing_timeout_id);
 		editor->details->typing_timeout_id = 0;
 	}
@@ -232,11 +211,10 @@ typing_timeout_cb (gpointer user_data)
 	NemoQueryEditor *editor;
 
 	editor = NEMO_QUERY_EDITOR (user_data);
+	editor->details->typing_timeout_id = 0;
 
 	nemo_query_editor_changed (editor);
 
-	editor->details->typing_timeout_id = 0;
-
 	return FALSE;
 }
 
@@ -249,7 +227,7 @@ entry_changed_cb (GtkWidget *entry, NemoQueryEditor *editor)
 		return;
 	}
 
-	if (editor->details->typing_timeout_id) {
+	if (editor->details->typing_timeout_id > 0) {
 		g_source_remove (editor->details->typing_timeout_id);
 	}
 
@@ -259,88 +237,6 @@ entry_changed_cb (GtkWidget *entry, NemoQueryEditor *editor)
 			       editor);
 }
 
-static void
-edit_clicked (GtkButton *button, NemoQueryEditor *editor)
-{
-	nemo_query_editor_set_visible (editor, TRUE);
-	nemo_query_editor_grab_focus (editor);
-}
-
-/* Location */
-
-static GtkWidget *
-location_row_create_widgets (NemoQueryEditorRow *row)
-{
-	GtkWidget *chooser;
-
-	chooser = gtk_file_chooser_button_new (_("Select folder to search in"),
-					       GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
-	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (chooser), TRUE);
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
-					     g_get_home_dir ());
-	gtk_widget_show (chooser);
-
-	g_signal_connect_swapped (chooser, "current-folder-changed",
-				  G_CALLBACK (nemo_query_editor_changed),
-				  row->editor);
-		
-	gtk_box_pack_start (GTK_BOX (row->hbox), chooser, FALSE, FALSE, 0);
-	
-	return chooser;
-}
-
-static void
-location_row_add_to_query (NemoQueryEditorRow *row,
-			   NemoQuery          *query)
-{
-	char *folder, *uri;
-	
-	folder = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (row->type_widget));
-	if (folder == NULL) {
-		/* I don't know why, but i got NULL here on initial search in browser mode
-		   even with the location set to the homedir in create_widgets... */
-		folder = g_strdup (g_get_home_dir ());
-	}
-	
-	uri = g_filename_to_uri (folder, NULL, NULL);
-	g_free (folder);
-		
-	nemo_query_set_location (query, uri);
-	g_free (uri);
-}
-
-static void
-location_row_free_data (NemoQueryEditorRow *row)
-{
-}
-
-static void
-location_add_rows_from_query (NemoQueryEditor    *editor,
-			      NemoQuery          *query)
-{
-	NemoQueryEditorRow *row;
-	char *uri, *folder;
-	
-	uri = nemo_query_get_location (query);
-
-	if (uri == NULL) {
-		return;
-	}
-	folder = g_filename_from_uri (uri, NULL, NULL);
-	g_free (uri);
-	if (folder == NULL) {
-		return;
-	}
-	
-	row = nemo_query_editor_add_row (editor,
-					     NEMO_QUERY_EDITOR_ROW_LOCATION);
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (row->type_widget),
-					     folder);
-	
-	g_free (folder);
-}
-
-
 /* Type */
 
 static gboolean
@@ -807,7 +703,6 @@ type_add_rows_from_query (NemoQueryEditor    *editor,
 
 		row = nemo_query_editor_add_row (editor,
 						     NEMO_QUERY_EDITOR_ROW_TYPE);
-		model = gtk_combo_box_get_model (GTK_COMBO_BOX (row->type_widget));
 		
 		type_add_custom_type (row, mime_type, desc, &iter);
 		gtk_combo_box_set_active_iter  (GTK_COMBO_BOX (row->type_widget),
@@ -850,7 +745,7 @@ remove_row_cb (GtkButton *clicked_button, NemoQueryEditorRow *row)
 	NemoQueryEditor *editor;
 
 	editor = row->editor;
-	gtk_container_remove (GTK_CONTAINER (editor->details->visible_vbox),
+	gtk_container_remove (GTK_CONTAINER (editor->details->vbox),
 			      row->hbox);
 	
 	editor->details->rows = g_list_remove (editor->details->rows, row);
@@ -908,7 +803,7 @@ nemo_query_editor_add_row (NemoQueryEditor *editor,
 	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 	row->hbox = hbox;
 	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (editor->details->visible_vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (editor->details->vbox), hbox, FALSE, FALSE, 0);
 
 	combo = gtk_combo_box_text_new ();
 	row->combo = combo;
@@ -945,12 +840,6 @@ nemo_query_editor_add_row (NemoQueryEditor *editor,
 	return row;
 }
 
-static void
-go_search_cb (GtkButton *clicked_button, NemoQueryEditor *editor)
-{
-	nemo_query_editor_changed_force (editor, TRUE);
-}
-
 static void
 add_new_row_cb (GtkButton *clicked_button, NemoQueryEditor *editor)
 {
@@ -961,58 +850,44 @@ add_new_row_cb (GtkButton *clicked_button, NemoQueryEditor *editor)
 static void
 nemo_query_editor_init (NemoQueryEditor *editor)
 {
-	GtkWidget *hbox, *label, *button;
-	char *label_markup;
-
 	editor->details = G_TYPE_INSTANCE_GET_PRIVATE (editor, NEMO_TYPE_QUERY_EDITOR,
 						       NemoQueryEditorDetails);
-	editor->details->is_visible = TRUE;
+	editor->details->is_visible = FALSE;
+    editor->details->search_bar_revealer = gtk_revealer_new ();
+
+	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (editor)),
+				     GTK_STYLE_CLASS_TOOLBAR);
+	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (editor)),
+				     GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
 
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (editor), GTK_ORIENTATION_VERTICAL);
 
-	editor->details->invisible_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (editor->details->invisible_vbox), 6);
-	gtk_box_pack_start (GTK_BOX (editor), editor->details->invisible_vbox,
+	editor->details->vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+    gtk_container_add (GTK_CONTAINER (editor->details->search_bar_revealer), editor->details->vbox);
+    gtk_widget_show_all (editor->details->search_bar_revealer);
+	gtk_widget_set_no_show_all (editor->details->vbox, TRUE);
+	gtk_container_set_border_width (GTK_CONTAINER (editor->details->vbox), 6);
+	gtk_box_pack_start (GTK_BOX (editor), editor->details->search_bar_revealer,
 			    FALSE, FALSE, 0);
-	editor->details->visible_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (editor->details->visible_vbox), 6);
-	gtk_box_pack_start (GTK_BOX (editor), editor->details->visible_vbox,
-			    FALSE, FALSE, 0);
-	/* Only show visible vbox */
-	gtk_widget_show (editor->details->visible_vbox);
-
-	/* Create invisible part: */
-	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-	gtk_box_pack_start (GTK_BOX (editor->details->invisible_vbox),
-			    hbox, FALSE, FALSE, 0);
-	gtk_widget_show (hbox);
-	
-	label = gtk_label_new ("");
-	label_markup = g_strconcat ("<b>", _("Search Folder"), "</b>", NULL);
-	gtk_label_set_markup (GTK_LABEL (label), label_markup);
-	g_free (label_markup);
-
-	gtk_style_context_add_class (gtk_widget_get_style_context (label),
-				     "nemo-cluebar-label");
-	
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	gtk_widget_show (label);
-	
-	button = gtk_button_new_with_label (_("Edit"));
-	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-	gtk_widget_show (button);
-
-	g_signal_connect (button, "clicked",
-			  G_CALLBACK (edit_clicked), editor);
-
-	gtk_widget_set_tooltip_text (button,
-				     _("Edit the saved search"));
 }
 
 void
 nemo_query_editor_set_default_query (NemoQueryEditor *editor)
 {
-	nemo_query_editor_add_row (editor, NEMO_QUERY_EDITOR_ROW_LOCATION);
+	nemo_query_editor_changed (editor);
+}
+
+static void
+on_all_button_toggled (GtkToggleButton     *button,
+		       NemoQueryEditor *editor)
+{
+	nemo_query_editor_changed (editor);
+}
+
+static void
+on_current_button_toggled (GtkToggleButton     *button,
+			   NemoQueryEditor *editor)
+{
 	nemo_query_editor_changed (editor);
 }
 
@@ -1037,46 +912,45 @@ finish_first_line (NemoQueryEditor *editor, GtkWidget *hbox, gboolean use_go)
 	gtk_widget_set_tooltip_text (button,
 				     _("Add a new criterion to this search"));
 
-	if (use_go) {
-		button = gtk_button_new_with_label (_("Go"));
-	} else {
-		button = gtk_button_new_with_label (_("Reload"));
-	}
-	gtk_widget_show (button);
-
-	gtk_widget_set_tooltip_text (button,
-				     _("Perform or update the search"));
-		
-	g_signal_connect (button, "clicked",
-			  G_CALLBACK (go_search_cb), editor);
-		
-	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	editor->details->search_current_button = gtk_radio_button_new_with_label (NULL, _("Current"));
+	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (editor->details->search_current_button), FALSE);
+	gtk_widget_show (editor->details->search_current_button);
+	editor->details->search_all_button = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (editor->details->search_current_button),
+											  _("All Files"));
+	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (editor->details->search_all_button), FALSE);
+	gtk_widget_show (editor->details->search_all_button);
+	g_signal_connect (editor->details->search_all_button, "toggled",
+			  G_CALLBACK (on_all_button_toggled), editor);
+	g_signal_connect (editor->details->search_current_button, "toggled",
+			  G_CALLBACK (on_current_button_toggled), editor);
+	gtk_box_pack_start (GTK_BOX (hbox), editor->details->search_current_button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), editor->details->search_all_button, FALSE, FALSE, 0);
 }
 
 static void
-setup_internal_entry (NemoQueryEditor *editor)
+setup_widgets (NemoQueryEditor *editor)
 {
-	GtkWidget *hbox, *label;
-	char *label_markup;
-	
+	GtkWidget *hbox;
+
 	/* Create visible part: */
 	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (editor->details->visible_vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (editor->details->vbox), hbox, FALSE, FALSE, 0);
 
-	label = gtk_label_new ("");
-	label_markup = g_strconcat ("<b>", _("_Search for:"), "</b>", NULL);
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), label_markup);
-	g_free (label_markup);
-	gtk_widget_show (label);
+#if GTK_CHECK_VERSION(3,6,0)
+	editor->details->entry = gtk_search_entry_new ();
+#else
+    GtkWidget *label = gtk_label_new ("");
+    char *label_markup = g_strconcat ("<b>", _("_Search for:"), "</b>", NULL);
+    gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), label_markup);
+    g_free (label_markup);
+    gtk_widget_show (label);
 
-	gtk_style_context_add_class (gtk_widget_get_style_context (label),
-				     "nemo-cluebar-label");
-	
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
-	editor->details->entry = gtk_entry_new ();
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), editor->details->entry);
+    editor->details->entry = gtk_entry_new ();
+    gtk_label_set_mnemonic_widget (GTK_LABEL (label), editor->details->entry);
+#endif
 	gtk_box_pack_start (GTK_BOX (hbox), editor->details->entry, TRUE, TRUE, 0);
 
 	g_signal_connect (editor->details->entry, "activate",
@@ -1088,49 +962,15 @@ setup_internal_entry (NemoQueryEditor *editor)
 	finish_first_line (editor, hbox, TRUE);
 }
 
-static void
-setup_external_entry (NemoQueryEditor *editor, GtkWidget *entry)
-{
-	GtkWidget *hbox, *label;
-	gchar *label_markup;
-	
-	/* Create visible part: */
-	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (editor->details->visible_vbox), hbox, FALSE, FALSE, 0);
-
-	label_markup = g_strconcat ("<b>", _("Search results"), "</b>", NULL);
-	label = gtk_label_new (NULL);
-	gtk_label_set_markup (GTK_LABEL (label), label_markup);
-	gtk_widget_show (label);
-
-	gtk_style_context_add_class (gtk_widget_get_style_context (label),
-				     "nemo-cluebar-label");
-	
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	
-	editor->details->entry = entry;
-	g_signal_connect (editor->details->entry, "activate",
-			  G_CALLBACK (entry_activate_cb), editor);
-	g_signal_connect (editor->details->entry, "changed",
-			  G_CALLBACK (entry_changed_cb), editor);
-
-	finish_first_line (editor, hbox, FALSE);
-
-	g_free (label_markup);
-}
-
 void
 nemo_query_editor_set_visible (NemoQueryEditor *editor,
 				   gboolean visible)
 {
 	editor->details->is_visible = visible;
 	if (visible) {
-		gtk_widget_show (editor->details->visible_vbox);
-		gtk_widget_hide (editor->details->invisible_vbox);
+		gtk_revealer_set_reveal_child (GTK_REVEALER (editor->details->search_bar_revealer), TRUE);
 	} else {
-		gtk_widget_hide (editor->details->visible_vbox);
-		gtk_widget_show (editor->details->invisible_vbox);
+		gtk_revealer_set_reveal_child (GTK_REVEALER (editor->details->search_bar_revealer), FALSE);
 	}
 }
 
@@ -1164,7 +1004,7 @@ nemo_query_editor_changed_force (NemoQueryEditor *editor, gboolean force_reload)
 static void
 nemo_query_editor_changed (NemoQueryEditor *editor)
 {
-	nemo_query_editor_changed_force (editor, FALSE);
+	nemo_query_editor_changed_force (editor, TRUE);
 }
 
 void
@@ -1175,6 +1015,23 @@ nemo_query_editor_grab_focus (NemoQueryEditor *editor)
 	}
 }
 
+static void
+add_location_to_query (NemoQueryEditor *editor,
+		       NemoQuery       *query)
+{
+	char *uri;
+
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (editor->details->search_all_button))) {
+		uri = nemo_get_home_directory_uri ();
+	} else {
+		uri = g_strdup (editor->details->current_uri);
+	}
+
+	nemo_query_set_location (query, uri);
+	g_free (uri);
+}
+
+
 NemoQuery *
 nemo_query_editor_get_query (NemoQueryEditor *editor)
 {
@@ -1197,6 +1054,8 @@ nemo_query_editor_get_query (NemoQueryEditor *editor)
 	query = nemo_query_new ();
 	nemo_query_set_text (query, query_text);
 
+	add_location_to_query (editor, query);
+
 	for (l = editor->details->rows; l != NULL; l = l->next) {
 		row = l->data;
 		
@@ -1219,85 +1078,49 @@ nemo_query_editor_clear_query (NemoQueryEditor *editor)
 }
 
 GtkWidget *
-nemo_query_editor_new (gboolean start_hidden)
+nemo_query_editor_new (void)
 {
 	GtkWidget *editor;
 
 	editor = g_object_new (NEMO_TYPE_QUERY_EDITOR, NULL);
-	nemo_query_editor_set_visible (NEMO_QUERY_EDITOR (editor),
-					   !start_hidden);
-	
-	setup_internal_entry (NEMO_QUERY_EDITOR (editor));
+	setup_widgets (NEMO_QUERY_EDITOR (editor));
 		
 	return editor;
 }
 
 static void
-detach_from_external_entry (NemoQueryEditor *editor)
+update_location (NemoQueryEditor *editor,
+		 NemoQuery       *query)
 {
-	if (editor->details->bar != NULL) {
-		nemo_search_bar_return_entry (editor->details->bar);
-		g_signal_handlers_block_by_func (editor->details->entry,
-						 entry_activate_cb,
-						 editor);
-		g_signal_handlers_block_by_func (editor->details->entry,
-						 entry_changed_cb,
-						 editor);
+	char *uri;
+	NemoFile *file;
+
+	uri = nemo_query_get_location (query);
+	if (uri == NULL) {
+		return;
 	}
-}
+	g_free (editor->details->current_uri);
+	editor->details->current_uri = uri;
+	file = nemo_file_get_by_uri (uri);
 
-static void
-attach_to_external_entry (NemoQueryEditor *editor)
-{
-	if (editor->details->bar != NULL) {
-		nemo_search_bar_borrow_entry (editor->details->bar);
-		g_signal_handlers_unblock_by_func (editor->details->entry,
-						   entry_activate_cb,
-						   editor);
-		g_signal_handlers_unblock_by_func (editor->details->entry,
-						   entry_changed_cb,
-						   editor);
+	if (file != NULL) {
+		char *name;
+		if (nemo_file_is_home (file)) {
+			name = g_strdup (_("Home"));
+		} else {
+			char *filename;
+			filename = nemo_file_get_display_name (file);
+			name = g_strdup_printf ("\342\200\234%s\342\200\235", filename);
+			g_free (filename);
+		}
+		gtk_button_set_label (GTK_BUTTON (editor->details->search_current_button),
+				      name);
+		g_free (name);
 
-		editor->details->change_frozen = TRUE;
-		gtk_entry_set_text (GTK_ENTRY (editor->details->entry),
-				    editor->details->last_set_query_text);
-		editor->details->change_frozen = FALSE;
+		nemo_file_unref (file);
 	}
 }
 
-GtkWidget*
-nemo_query_editor_new_with_bar (gboolean start_hidden,
-				    gboolean start_attached,
-				    NemoSearchBar *bar,
-				    NemoWindowSlot *slot)
-{
-	GtkWidget *entry;
-	NemoQueryEditor *editor;
-
-	editor = NEMO_QUERY_EDITOR (g_object_new (NEMO_TYPE_QUERY_EDITOR, NULL));
-	nemo_query_editor_set_visible (editor, !start_hidden);
-
-	editor->details->bar = bar;
-	eel_add_weak_pointer (&editor->details->bar);
-
-	editor->details->slot = slot;
-
-	entry = nemo_search_bar_borrow_entry (bar);
-	setup_external_entry (editor, entry);
-	if (!start_attached) {
-		detach_from_external_entry (editor);
-	}
-
-	g_signal_connect_object (slot, "active",
-				 G_CALLBACK (attach_to_external_entry),
-				 editor, G_CONNECT_SWAPPED);
-	g_signal_connect_object (slot, "inactive",
-				 G_CALLBACK (detach_from_external_entry),
-				 editor, G_CONNECT_SWAPPED);
-	
-	return GTK_WIDGET (editor);
-}
-
 void
 nemo_query_editor_set_query (NemoQueryEditor *editor, NemoQuery *query)
 {
@@ -1318,6 +1141,8 @@ nemo_query_editor_set_query (NemoQueryEditor *editor, NemoQuery *query)
 	editor->details->change_frozen = TRUE;
 	gtk_entry_set_text (GTK_ENTRY (editor->details->entry), text);
 
+        update_location (editor, query);
+
 	for (type = 0; type < NEMO_QUERY_EDITOR_ROW_LAST; type++) {
 		row_type[type].add_rows_from_query (editor, query);
 	}
