@@ -32,13 +32,14 @@
 #include "nemo-application.h"
 #include "nemo-floating-bar.h"
 #include "nemo-location-bar.h"
-#include "nemo-search-bar.h"
 #include "nemo-pathbar.h"
 #include "nemo-window-private.h"
 #include "nemo-window-slot.h"
 #include "nemo-trash-bar.h"
 #include "nemo-view-factory.h"
 #include "nemo-x-content-bar.h"
+#include "nemo-interesting-folder-bar.h"
+#include "nemo-thumbnail-problem-bar.h"
 #include <eel/eel-accessibility.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-glib-extensions.h>
@@ -117,14 +118,14 @@ set_displayed_location (NemoWindowSlot *slot, GFile *location)
                 g_object_unref (bookmark_location);
         }
         
-        if (recreate) {
-                /* We've changed locations, must recreate bookmark for current location. */
-		g_clear_object (&slot->last_location_bookmark);
+    if (recreate) {
+        /* We've changed locations, must recreate bookmark for current location. */
+        g_clear_object (&slot->last_location_bookmark);
 
-		slot->last_location_bookmark = slot->current_location_bookmark;
-		slot->current_location_bookmark = (location == NULL) ? NULL
-                        : nemo_bookmark_new (location, NULL, NULL);
-        }
+        slot->last_location_bookmark = slot->current_location_bookmark;
+        slot->current_location_bookmark = (location == NULL) ?
+                            NULL : nemo_bookmark_new (location, NULL, NULL);
+    }
 }
 
 static void
@@ -320,7 +321,7 @@ viewed_file_changed_callback (NemoFile *file,
 					nemo_path_bar_clear_buttons (NEMO_PATH_BAR (slot->pane->path_bar));
 				}
 				
-				nemo_window_slot_go_to (slot, go_to_file, FALSE);
+				nemo_window_slot_open_location (slot, go_to_file, 0);
 				g_object_unref (go_to_file);
 			} else {
 				nemo_window_slot_go_home (slot, FALSE);
@@ -845,8 +846,8 @@ got_file_info_for_view_selection_callback (NemoFile *file,
 		mimetype = nemo_file_get_mime_type (file);
 
 		/* Look in metadata for view */
-		view_id = nemo_file_get_metadata 
-			(file, NEMO_METADATA_KEY_DEFAULT_VIEW, NULL);
+		view_id = nemo_global_preferences_get_ignore_view_metadata () ? g_strdup (nemo_window_get_ignore_meta_view_id (window)) :
+                                                                        nemo_file_get_metadata (file, NEMO_METADATA_KEY_DEFAULT_VIEW, NULL);
 		if (view_id != NULL && 
 		    !nemo_view_factory_view_supports_uri (view_id,
 							      location,
@@ -858,7 +859,15 @@ got_file_info_for_view_selection_callback (NemoFile *file,
 
 		/* Otherwise, use default */
 		if (view_id == NULL) {
-			view_id = nemo_global_preferences_get_default_folder_viewer_preference_as_iid ();
+            gchar *name;
+            name = nemo_file_get_name (file);
+
+            if (g_strcmp0 (name, "x-nemo-search") == 0)
+                view_id = g_strdup (NEMO_LIST_VIEW_IID);
+            else
+                view_id = nemo_global_preferences_get_default_folder_viewer_preference_as_iid ();
+
+            g_free (name);
 
 			if (view_id != NULL && 
 			    !nemo_view_factory_view_supports_uri (view_id,
@@ -905,7 +914,7 @@ got_file_info_for_view_selection_callback (NemoFile *file,
 
 						root = g_file_new_for_path ("/");
 						/* the last fallback is to go to a known place that can't be deleted! */
-						nemo_window_slot_go_to (slot, location, FALSE);
+						nemo_window_slot_open_location (slot, location, 0);
 						g_object_unref (root);
 					}
 				} else {
@@ -926,7 +935,7 @@ got_file_info_for_view_selection_callback (NemoFile *file,
 			/* We're missing a previous location (if opened location
 			 * in a new tab) so close it and return */
 			if (slot->location == NULL) {
-				nemo_window_pane_slot_close (slot->pane, slot);
+				nemo_window_pane_close_slot (slot->pane, slot);
 			} else {
 				/* We disconnected this, so we need to re-connect it */
 				viewed_file = nemo_file_get (slot->location);
@@ -1093,6 +1102,23 @@ nemo_window_emit_location_change (NemoWindow *window,
 	g_free (uri);
 }
 
+static void
+nemo_window_slot_emit_location_change (NemoWindowSlot *slot,
+					   GFile *from,
+					   GFile *to)
+{
+	char *from_uri = NULL;
+	char *to_uri = NULL;
+
+	if (from != NULL)
+		from_uri = g_file_get_uri (from);
+	if (to != NULL)
+		to_uri = g_file_get_uri (to);
+	g_signal_emit_by_name (slot, "location-changed", from_uri, to_uri);
+	g_free (to_uri);
+	g_free (from_uri);
+}
+
 /* reports location change to window's "loading-uri" clients, i.e.
  * sidebar panels [used when switching tabs]. It will emit the pending
  * location, or the existing location if none is pending.
@@ -1291,6 +1317,18 @@ nemo_window_slot_show_trash_bar (NemoWindowSlot *slot)
 	nemo_window_slot_add_extra_location_widget (slot, bar);
 }
 
+static void
+maybe_show_interesting_folder_bar (NemoWindowSlot *slot)
+{
+    GtkWidget *bar = nemo_interesting_folder_bar_new_for_location (nemo_window_slot_get_current_view(slot),
+                                                                   slot->location);
+
+    if (bar) {
+        gtk_widget_show (bar);
+        nemo_window_slot_add_extra_location_widget (slot, bar);
+    }
+}
+
 typedef struct {
 	NemoWindowSlot *slot;
 	GCancellable *cancellable;
@@ -1375,7 +1413,9 @@ update_for_new_location (NemoWindowSlot *slot)
 	location_really_changed =
 		slot->location == NULL ||
 		!g_file_equal (slot->location, new_location);
-		
+
+	nemo_window_slot_emit_location_change (slot, slot->location, new_location);
+
         /* Set the new location. */
 	g_clear_object (&slot->location);
 	slot->location = new_location;
@@ -1410,12 +1450,14 @@ update_for_new_location (NemoWindowSlot *slot)
 		
 		directory = nemo_directory_get (slot->location);
 
-		nemo_window_slot_update_query_editor (slot);
-
 		if (nemo_directory_is_in_trash (directory)) {
 			nemo_window_slot_show_trash_bar (slot);
 		}
 
+        maybe_show_interesting_folder_bar (slot);
+
+        nemo_window_slot_check_bad_cache_bar (slot);
+
 		/* need the mount to determine if we should put up the x-content cluebar */
 		if (slot->find_mount_cancellable != NULL) {
 			g_cancellable_cancel (slot->find_mount_cancellable);
@@ -1724,17 +1766,22 @@ nemo_window_slot_set_content_view (NemoWindowSlot *slot,
         end_location_change (slot);
 
 	file = nemo_file_get (slot->location);
-	nemo_file_set_metadata 
-		(file, NEMO_METADATA_KEY_DEFAULT_VIEW, NULL, id);
-        nemo_file_unref (file);
-        
-        nemo_window_slot_set_allow_stop (slot, TRUE);
 
-        if (nemo_view_get_selection_count (slot->content_view) == 0) {
-                /* If there is no selection, queue a scroll to the same icon that
-                 * is currently visible */
-                slot->pending_scroll_to = nemo_view_get_first_visible_file (slot->content_view);
-        }
+    if (nemo_global_preferences_get_ignore_view_metadata ()) {
+        nemo_window_set_ignore_meta_view_id (nemo_window_slot_get_window (slot), id);
+    } else {
+        nemo_file_set_metadata (file, NEMO_METADATA_KEY_DEFAULT_VIEW, NULL, id);
+    }
+
+    nemo_file_unref (file);
+
+    nemo_window_slot_set_allow_stop (slot, TRUE);
+
+    if (nemo_view_get_selection_count (slot->content_view) == 0) {
+            /* If there is no selection, queue a scroll to the same icon that
+             * is currently visible */
+            slot->pending_scroll_to = nemo_view_get_first_visible_file (slot->content_view);
+    }
 	slot->location_change_type = NEMO_LOCATION_CHANGE_RELOAD;
 	
         create_content_view (slot, id);
@@ -1756,7 +1803,7 @@ void
 nemo_window_back_or_forward (NemoWindow *window, 
 				 gboolean back,
 				 guint distance,
-				 gboolean new_tab)
+				 NemoWindowOpenFlags flags)
 {
 	NemoWindowSlot *slot;
 	GList *list;
@@ -1782,10 +1829,8 @@ nemo_window_back_or_forward (NemoWindow *window,
         bookmark = g_list_nth_data (list, distance);
 	location = nemo_bookmark_get_location (bookmark);
 
-	if (new_tab) {
-		nemo_window_slot_open_location_full (slot, location,
-							 NEMO_WINDOW_OPEN_FLAG_NEW_TAB,
-							 NULL, NULL, NULL);
+	if (flags != 0) {
+		nemo_window_slot_open_location (slot, location, flags);
 	} else {
 		char *scroll_pos;
 
@@ -1838,3 +1883,28 @@ nemo_window_slot_reload (NemoWindowSlot *slot)
 	g_object_unref (location);
 	g_list_free_full (selection, g_object_unref);
 }
+
+void
+nemo_window_slot_check_bad_cache_bar (NemoWindowSlot *slot)
+{
+    if (NEMO_IS_DESKTOP_WINDOW (nemo_window_slot_get_window (slot)))
+        return;
+
+    if (nemo_application_get_cache_bad (nemo_application_get_singleton ()) &&
+        !nemo_application_get_cache_problem_ignored (nemo_application_get_singleton ())) {
+        if (slot->cache_bar != NULL) {
+            gtk_widget_show (slot->cache_bar);
+        } else {
+            GtkWidget *bad_bar = nemo_thumbnail_problem_bar_new (nemo_window_slot_get_current_view (slot));
+            if (bad_bar) {
+                gtk_widget_show (bad_bar);
+                nemo_window_slot_add_extra_location_widget (slot, bad_bar);
+                slot->cache_bar = bad_bar;
+            }
+        }
+    } else {
+        if (slot->cache_bar != NULL) {
+            gtk_widget_hide (slot->cache_bar);
+        }
+    }
+}
