@@ -68,10 +68,8 @@ widget_is_in_temporary_bars (GtkWidget *widget,
 {
 	gboolean res = FALSE;
 
-	if ((gtk_widget_get_ancestor (widget, NEMO_TYPE_LOCATION_BAR) != NULL &&
-	     pane->temporary_navigation_bar) ||
-	    (gtk_widget_get_ancestor (widget, NEMO_TYPE_SEARCH_BAR) != NULL &&
-	     pane->temporary_search_bar))
+	if (gtk_widget_get_ancestor (widget, NEMO_TYPE_LOCATION_BAR) != NULL &&
+	     pane->temporary_navigation_bar)
 		res = TRUE;
 
 	return res;
@@ -182,42 +180,6 @@ bookmark_list_get_uri_index (GList *list, GFile *location)
 	return -1;
 }
 
-static void
-search_bar_activate_callback (NemoSearchBar *bar,
-			      NemoWindowPane *pane)
-{
-	char *uri, *current_uri;
-	NemoDirectory *directory;
-	NemoSearchDirectory *search_directory;
-	NemoQuery *query;
-	GFile *location;
-
-	uri = nemo_search_directory_generate_new_uri ();
-	location = g_file_new_for_uri (uri);
-
-	directory = nemo_directory_get (location);
-	g_assert (NEMO_IS_SEARCH_DIRECTORY (directory));
-
-	search_directory = NEMO_SEARCH_DIRECTORY (directory);
-	query = nemo_search_bar_get_query (NEMO_SEARCH_BAR (pane->search_bar));
-
-	if (query != NULL) {
-		current_uri = nemo_window_slot_get_location_uri (pane->active_slot);
-
-		nemo_query_set_location (query, current_uri);
-		nemo_search_directory_set_query (search_directory, query);
-
-		g_free (current_uri);
-		g_object_unref (query);
-	}
-
-	nemo_window_slot_go_to (pane->active_slot, location, FALSE);
-
-	nemo_directory_unref (directory);
-	g_object_unref (location);
-	g_free (uri);
-}
-
 static void
 nemo_window_pane_hide_temporary_bars (NemoWindowPane *pane)
 {
@@ -236,7 +198,6 @@ nemo_window_pane_hide_temporary_bars (NemoWindowPane *pane)
 		 */
 		if (NEMO_IS_SEARCH_DIRECTORY (directory)) {
 			nemo_toolbar_set_show_main_bar (NEMO_TOOLBAR (pane->tool_bar), FALSE);
-			nemo_toolbar_set_show_search_bar (NEMO_TOOLBAR (pane->tool_bar), TRUE);
 		} else {
 			gtk_widget_hide (pane->tool_bar);
 		}
@@ -245,18 +206,6 @@ nemo_window_pane_hide_temporary_bars (NemoWindowPane *pane)
 	}
 }
 
-static void
-search_bar_cancel_callback (GtkWidget *widget,
-			    NemoWindowPane *pane)
-{
-	GtkAction *search;
-
-	search = gtk_action_group_get_action (pane->action_group,
-					      NEMO_ACTION_SEARCH);
-
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (search), FALSE);
-}
-
 static void
 location_entry_changed_cb (NemoToolbar *toolbar, gboolean val, gpointer data)
 {
@@ -278,53 +227,20 @@ navigation_bar_cancel_callback (GtkWidget *widget,
 	restore_focus_widget (pane);
 }
 
-static void
-nemo_window_pane_ensure_search_bar (NemoWindowPane *pane)
-{
-	remember_focus_widget (pane);
-
-	nemo_toolbar_set_show_search_bar (NEMO_TOOLBAR (pane->tool_bar), TRUE);
-
-	if (!g_settings_get_boolean (nemo_window_state,
-				     NEMO_WINDOW_STATE_START_WITH_TOOLBAR)) {
-		nemo_toolbar_set_show_main_bar (NEMO_TOOLBAR (pane->tool_bar), FALSE);
-		gtk_widget_show (pane->tool_bar);
-		nemo_search_bar_clear (NEMO_SEARCH_BAR (pane->search_bar));
-
-		pane->temporary_search_bar = TRUE;
-	}
-
-	nemo_search_bar_grab_focus (NEMO_SEARCH_BAR (pane->search_bar));
-}
-
-static void
-nemo_window_pane_hide_search_bar (NemoWindowPane *pane)
-{
-	nemo_toolbar_set_show_search_bar (NEMO_TOOLBAR (pane->tool_bar), FALSE);
-	restore_focus_widget (pane);
-
-	if (pane->temporary_search_bar) {
-		pane->temporary_search_bar = FALSE;
-
-		gtk_widget_hide (pane->tool_bar);
-	}
-}
-
 static void
 navigation_bar_location_changed_callback (GtkWidget *widget,
-					  const char *uri,
-					  NemoWindowPane *pane)
+                                        const gchar *uri,
+                                     NemoWindowPane *pane)
 {
-	GFile *location;
+    GFile *location;
 
-	nemo_window_pane_hide_search_bar (pane);
-	nemo_window_pane_hide_temporary_bars (pane);
+    nemo_window_pane_hide_temporary_bars (pane);
 
-	restore_focus_widget (pane);
+    restore_focus_widget (pane);
 
-	location = g_file_new_for_uri (uri);
-	nemo_window_slot_go_to (pane->active_slot, location, FALSE);
-	g_object_unref (location);
+    location = g_file_new_for_uri (uri);
+    nemo_window_slot_open_location (pane->active_slot, location, 0);
+    g_object_unref (location);
 }
 
 static gboolean
@@ -352,9 +268,9 @@ path_bar_location_changed_callback (GtkWidget *widget,
 	/* check whether we already visited the target location */
 	i = bookmark_list_get_uri_index (slot->back_list, location);
 	if (i >= 0) {
-		nemo_window_back_or_forward (pane->window, TRUE, i, FALSE);
+		nemo_window_back_or_forward (pane->window, TRUE, i, 0);
 	} else {
-		nemo_window_slot_go_to (pane->active_slot, location, FALSE);
+		nemo_window_slot_open_location (pane->active_slot, location, 0);
 	}
 }
 
@@ -422,8 +338,7 @@ path_bar_button_released_callback (GtkWidget *widget,
 
 		if (flags != 0) {
 			slot = nemo_window_get_active_slot (pane->window);
-			nemo_window_slot_open_location (slot, location,
-							    flags, NULL);
+			nemo_window_slot_open_location (slot, location, flags);
 			g_object_unref (location);
 			return TRUE;
 		}
@@ -815,31 +730,21 @@ action_show_hide_search_callback (GtkAction *action,
 {
 	NemoWindowPane *pane = user_data;
 	NemoWindow *window = pane->window;
+	NemoWindowSlot *slot;
+
+	slot = pane->active_slot;
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action))) {
-		nemo_window_pane_ensure_search_bar (pane);
+	    remember_focus_widget (pane);
+	    nemo_window_slot_set_query_editor_visible (slot, TRUE);
 	} else {
-		NemoWindowSlot *slot;
 		GFile *location = NULL;
 
-		slot = pane->active_slot;
-		nemo_window_pane_hide_search_bar (pane);
+		restore_focus_widget (pane);
 
 		/* Use the location bar as the return location */
 		if (slot->query_editor != NULL) {
-			NemoQuery *query;
-			char *uri;
-
-			query = nemo_query_editor_get_query (slot->query_editor);
-			if (query != NULL) {
-				uri = nemo_query_get_location (query);
-				if (uri != NULL) {
-					location = g_file_new_for_uri (uri);
-					g_free (uri);
-				}
-				g_object_unref (query);
-			}
-
+		    location = nemo_query_editor_get_location (slot->query_editor);
 			/* Last try: use the home directory as the return location */
 			if (location == NULL) {
 				location = g_file_new_for_path (g_get_home_dir ());
@@ -848,6 +753,8 @@ action_show_hide_search_callback (GtkAction *action,
 			nemo_window_go_to (window, location);
 			g_object_unref (location);
 		}
+
+		nemo_window_slot_set_query_editor_visible (slot, FALSE);
 	}
 }
 
@@ -948,7 +855,9 @@ nemo_window_pane_constructed (GObject *obj)
 
 	pane->action_group = action_group;
 
-	setup_search_action (pane);
+    if (!NEMO_IS_DESKTOP_WINDOW (window))
+        setup_search_action (pane);
+
 	g_signal_connect (pane->action_group, "pre-activate",
 			  G_CALLBACK (toolbar_action_group_activated_callback), pane);
 
@@ -993,16 +902,6 @@ nemo_window_pane_constructed (GObject *obj)
 	g_signal_connect_object (nemo_location_bar_get_entry (NEMO_LOCATION_BAR (pane->location_bar)), "focus-in-event",
 				 G_CALLBACK (toolbar_focus_in_callback), pane, 0);
 
-	/* connect to the search bar signals */
-	pane->search_bar = nemo_toolbar_get_search_bar (NEMO_TOOLBAR (pane->tool_bar));
-
-	g_signal_connect_object (pane->search_bar, "activate",
-				 G_CALLBACK (search_bar_activate_callback), pane, 0);
-	g_signal_connect_object (pane->search_bar, "cancel",
-				 G_CALLBACK (search_bar_cancel_callback), pane, 0);
-	g_signal_connect_object (nemo_search_bar_get_entry (NEMO_SEARCH_BAR (pane->search_bar)), "focus-in-event",
-				 G_CALLBACK (toolbar_focus_in_callback), pane, 0);
-
 	/* initialize the notebook */
 	pane->notebook = g_object_new (NEMO_TYPE_NOTEBOOK, NULL);
 	gtk_box_pack_start (GTK_BOX (pane), pane->notebook,
@@ -1179,7 +1078,8 @@ nemo_window_pane_sync_location_widgets (NemoWindowPane *pane)
 }
 
 static void
-toggle_toolbar_search_button (NemoWindowPane *pane)
+toggle_toolbar_search_button (NemoWindowPane *pane,
+                                  gboolean        active)
 {
 	GtkActionGroup *group;
 	GtkAction *action;
@@ -1187,11 +1087,7 @@ toggle_toolbar_search_button (NemoWindowPane *pane)
 	group = pane->action_group;
 	action = gtk_action_group_get_action (group, NEMO_ACTION_SEARCH);
 
-	g_signal_handlers_block_by_func (action,
-					 action_show_hide_search_callback, pane);
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
-	g_signal_handlers_unblock_by_func (action,
-					   action_show_hide_search_callback, pane);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), active);
 }
 
 void
@@ -1210,14 +1106,9 @@ nemo_window_pane_sync_search_widgets (NemoWindowPane *pane)
 	}
 
 	if (search_directory != NULL) {
-		if (!nemo_search_directory_is_saved_search (search_directory)) {
-			nemo_toolbar_set_show_search_bar (NEMO_TOOLBAR (pane->tool_bar), TRUE);
-			pane->temporary_search_bar = FALSE;
-		} else {
-			toggle_toolbar_search_button (pane);
-		}
+		toggle_toolbar_search_button (pane, TRUE);
 	} else {
-		search_bar_cancel_callback (pane->search_bar, pane);
+	    toggle_toolbar_search_button (pane, FALSE);
 	}
 
 	nemo_directory_unref (directory);
