@@ -38,7 +38,6 @@
 #include "nemo-mime-actions.h"
 #include "nemo-notebook.h"
 #include "nemo-places-sidebar.h"
-#include "nemo-search-bar.h"
 #include "nemo-tree-sidebar.h"
 #include "nemo-view-factory.h"
 #include "nemo-window-manage-views.h"
@@ -92,6 +91,7 @@ static void mouse_back_button_changed		     (gpointer                  callback_
 static void mouse_forward_button_changed	     (gpointer                  callback_data);
 static void use_extra_mouse_buttons_changed          (gpointer              callback_data);
 static void side_pane_id_changed                    (NemoWindow            *window);
+static void handle_alt_menu_key                     (NemoWindow *window, gboolean on_release);
 
 /* Sanity check: highest mouse button value I could find was 14. 5 is our 
  * lower threshold (well-documented to be the one of the button events for the 
@@ -115,6 +115,8 @@ enum {
 	PROMPT_FOR_LOCATION,
 	LOADING_URI,
 	HIDDEN_FILES_MODE_CHANGED,
+	SLOT_ADDED,
+	SLOT_REMOVED,
 	LAST_SIGNAL
 };
 
@@ -169,22 +171,13 @@ nemo_window_push_status (NemoWindow *window,
 	}
 }
 
-void
-nemo_window_sync_status (NemoWindow *window)
-{
-	NemoWindowSlot *slot;
-
-	slot = nemo_window_get_active_slot (window);
-	nemo_window_push_status (window, slot->status_text);
-}
-
 void
 nemo_window_go_to (NemoWindow *window, GFile *location)
 {
 	g_return_if_fail (NEMO_IS_WINDOW (window));
 
-	nemo_window_slot_go_to (nemo_window_get_active_slot (window),
-				    location, FALSE);
+	nemo_window_slot_open_location (nemo_window_get_active_slot (window),
+					    location, 0);
 }
 
 void
@@ -195,17 +188,26 @@ nemo_window_go_to_full (NemoWindow *window,
 {
 	g_return_if_fail (NEMO_IS_WINDOW (window));
 
-	nemo_window_slot_go_to_full (nemo_window_get_active_slot (window),
-					 location, FALSE, callback, user_data);
+	nemo_window_slot_open_location_full (nemo_window_get_active_slot (window),
+						 location, 0, NULL, callback, user_data);
 }
 
-static gboolean
-nemo_window_go_up_signal (NemoWindow *window, gboolean close_behind)
+static void
+nemo_window_go_up_signal (NemoWindow *window)
 {
-	nemo_window_slot_go_up (nemo_window_get_active_slot (window),
-				    close_behind, FALSE);
+	nemo_window_slot_go_up (nemo_window_get_active_slot (window), 0);
+}
 
-	return TRUE;
+void 
+nemo_window_slot_removed (NemoWindow *window,  NemoWindowSlot *slot)
+{
+	g_signal_emit (window, signals[SLOT_REMOVED], 0, slot);
+}
+
+void 
+nemo_window_slot_added (NemoWindow *window,  NemoWindowSlot *slot)
+{
+    g_signal_emit (window, signals[SLOT_ADDED], 0, slot);
 }
 
 void
@@ -238,7 +240,7 @@ nemo_window_new_tab (NemoWindow *window)
 
 		new_slot = nemo_window_pane_open_slot (current_slot->pane, flags);
 		nemo_window_set_active_slot (window, new_slot);
-		nemo_window_slot_go_to (new_slot, location, FALSE);
+		nemo_window_slot_open_location (new_slot, location, 0);
 		g_object_unref (location);
 	}
 }
@@ -264,20 +266,24 @@ void
 nemo_window_sync_allow_stop (NemoWindow *window,
 				 NemoWindowSlot *slot)
 {
-	GtkAction *action;
+	GtkAction *stop_action;
+	GtkAction *reload_action;
 	gboolean allow_stop, slot_is_active;
 	NemoNotebook *notebook;
 
-	action = gtk_action_group_get_action (nemo_window_get_main_action_group (window),
-					      NEMO_ACTION_STOP);
-	allow_stop = gtk_action_get_sensitive (action);
+	stop_action = gtk_action_group_get_action (nemo_window_get_main_action_group (window),
+						   NEMO_ACTION_STOP);
+	reload_action = gtk_action_group_get_action (nemo_window_get_main_action_group (window),
+						     NEMO_ACTION_RELOAD);
+	allow_stop = gtk_action_get_sensitive (stop_action);
 
 	slot_is_active = (slot == nemo_window_get_active_slot (window));
 
 	if (!slot_is_active ||
 	    allow_stop != slot->allow_stop) {
 		if (slot_is_active) {
-			gtk_action_set_sensitive (action, slot->allow_stop);
+			gtk_action_set_visible (stop_action, slot->allow_stop);
+			gtk_action_set_visible (reload_action, !slot->allow_stop);
 		}
 
 		if (gtk_widget_get_realized (GTK_WIDGET (window))) {
@@ -298,10 +304,9 @@ nemo_window_prompt_for_location (NemoWindow *window,
 
 	g_return_if_fail (NEMO_IS_WINDOW (window));
 
-	pane = window->details->active_pane;
-	nemo_window_pane_ensure_location_bar (pane);
-
 	if (initial) {
+		nemo_window_show_location_entry(window);
+		pane = window->details->active_pane;
 		nemo_location_bar_set_location (NEMO_LOCATION_BAR (pane->location_bar),
 						    initial);
 	}
@@ -522,6 +527,18 @@ nemo_window_disable_chrome_mapping (GValue *value,
 	return TRUE;
 }
 
+static gboolean
+on_button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+{
+    NemoWindow *window = NEMO_WINDOW (user_data);
+    if (event->button == 3) {
+            /* simulate activating the menu */
+            handle_alt_menu_key (window, FALSE);
+            handle_alt_menu_key (window, TRUE);
+    }
+    return TRUE;
+}
+
 static void
 nemo_window_constructed (GObject *self)
 {
@@ -541,6 +558,7 @@ nemo_window_constructed (GObject *self)
 
 	G_OBJECT_CLASS (nemo_window_parent_class)->constructed (self);
 
+
 	grid = gtk_grid_new ();
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
 	gtk_widget_show (grid);
@@ -577,6 +595,10 @@ nemo_window_constructed (GObject *self)
 	toolbar_holder = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_container_add (GTK_CONTAINER (grid), toolbar_holder);
 	gtk_widget_show (toolbar_holder);
+
+    g_signal_connect_object (toolbar_holder, "button-press-event",
+                             G_CALLBACK (on_button_press_callback), window, 0);
+
 	window->details->toolbar_holder = toolbar_holder;
 
 	/* Register to menu provider extension signal managing menu updates */
@@ -613,12 +635,22 @@ nemo_window_constructed (GObject *self)
     gtk_container_add (GTK_CONTAINER (grid), sep);
     gtk_widget_show (sep);
 
-    gtk_container_add (GTK_CONTAINER (grid), nemo_statusbar);
+    GtkWidget *eb;
+
+    eb = gtk_event_box_new ();
+    gtk_container_add (GTK_CONTAINER (eb), nemo_statusbar);
+    gtk_container_add (GTK_CONTAINER (grid), eb);
+    gtk_widget_show (eb);
 
     window->details->statusbar = nemo_status_bar_get_real_statusbar (NEMO_STATUS_BAR (nemo_statusbar));
     window->details->help_message_cid = gtk_statusbar_get_context_id (GTK_STATUSBAR (window->details->statusbar),
                                                                       "help_message");
 
+    gtk_widget_add_events (GTK_WIDGET (eb), GDK_BUTTON_PRESS_MASK);
+
+    g_signal_connect_object (GTK_WIDGET (eb), "button-press-event",
+                             G_CALLBACK (on_button_press_callback), window, 0);
+
     g_settings_bind_with_mapping (nemo_window_state,
                       NEMO_WINDOW_STATE_START_WITH_STATUS_BAR,
                       window->details->nemo_status_bar,
@@ -643,6 +675,10 @@ nemo_window_constructed (GObject *self)
 
 	slot = nemo_window_pane_open_slot (window->details->active_pane, 0);
 	nemo_window_set_active_slot (window, slot);
+
+    if (g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_START_WITH_DUAL_PANE) &&
+        !window->details->disable_chrome)
+        nemo_window_split_view_on (window);
 }
 
 static void
@@ -753,7 +789,7 @@ nemo_window_finalize (GObject *object)
 	nemo_window_finalize_menus (window);
 
 	g_clear_object (&window->details->nav_state);
-	g_clear_object (&window->details->bookmark_list);
+
 	g_clear_object (&window->details->ui_manager);
 
 	g_free (window->details->sidebar_id);
@@ -863,7 +899,7 @@ nemo_window_close_pane (NemoWindow *window,
 	while (pane->slots != NULL) {
 		NemoWindowSlot *slot = pane->slots->data;
 
-		nemo_window_pane_close_slot (pane, slot);
+		nemo_window_pane_remove_slot_unsafe (pane, slot);
 	}
 
 	/* If the pane was active, set it to NULL. The caller is responsible
@@ -1067,6 +1103,7 @@ nemo_window_key_press_event (GtkWidget *widget,
 	NemoWindow *window;
 	NemoWindowSlot *active_slot;
 	NemoView *view;
+    GtkWidget *focus_widget;
 	int i;
 
 	window = NEMO_WINDOW (widget);
@@ -1085,6 +1122,17 @@ nemo_window_key_press_event (GtkWidget *widget,
 		}
 	}
 
+	focus_widget = gtk_window_get_focus (GTK_WINDOW (window));
+	if (view != NULL && focus_widget != NULL &&
+	    GTK_IS_EDITABLE (focus_widget)) {
+		/* if we have input focus on a GtkEditable (e.g. a GtkEntry), forward
+		 * the event to it before activating accelerator bindings too.
+		 */
+		if (gtk_window_propagate_key_event (GTK_WINDOW (window), event)) {
+			return TRUE;
+		}
+	}
+
 	for (i = 0; i < G_N_ELEMENTS (extra_window_keybindings); i++) {
 		if (extra_window_keybindings[i].keyval == event->keyval) {
 			const GList *action_groups;
@@ -1576,7 +1624,6 @@ nemo_window_sync_zoom_widgets (NemoWindow *window)
 		can_zoom_in = can_zoom && nemo_view_can_zoom_in (view);
 		can_zoom_out = can_zoom && nemo_view_can_zoom_out (view);
 	} else {
-		zoom_level = NEMO_ZOOM_LEVEL_STANDARD;
 		supports_zooming = FALSE;
 		can_zoom = FALSE;
 		can_zoom_in = FALSE;
@@ -1857,6 +1904,14 @@ nemo_window_get_extra_slot (NemoWindow *window)
 	return extra_pane->active_slot;
 }
 
+GList *
+nemo_window_get_panes (NemoWindow *window)
+{
+	g_assert (NEMO_IS_WINDOW (window));
+
+	return window->details->panes;
+}
+
 static void
 window_set_search_action_text (NemoWindow *window,
 			       gboolean setting)
@@ -1892,6 +1947,13 @@ create_extra_pane (NemoWindow *window)
 		gtk_paned_pack2 (paned, GTK_WIDGET (pane), TRUE, FALSE);
 	}
 
+    /* Make the paned think it's been manually resized, otherwise
+       things like the trash bar will force unwanted resizes */
+
+    int w;
+    w = gtk_widget_get_allocated_width (GTK_WIDGET (paned)) / 2;
+    gtk_paned_set_position (paned, w);
+
 	/* Ensure the toolbar doesn't pop itself into existence (double toolbars suck.) */
 	gtk_widget_hide (pane->tool_bar);
 
@@ -1927,16 +1989,12 @@ nemo_window_state_event (GtkWidget *widget,
 	return FALSE;
 }
 
-static void
-nemo_window_go_back (NemoWindow *window)
+static gboolean
+nemo_window_delete_event (GtkWidget *widget,
+			      GdkEventAny *event)
 {
-	nemo_window_back_or_forward (window, TRUE, 0, FALSE);
-}
-
-static void
-nemo_window_go_forward (NemoWindow *window)
-{
-	nemo_window_back_or_forward (window, FALSE, 0, FALSE);
+	nemo_window_close (NEMO_WINDOW (widget));
+	return FALSE;
 }
 
 static gboolean
@@ -1946,14 +2004,13 @@ nemo_window_button_press_event (GtkWidget *widget,
 	NemoWindow *window;
 	gboolean handled;
 
-	handled = FALSE;
 	window = NEMO_WINDOW (widget);
 
 	if (mouse_extra_buttons && (event->button == mouse_back_button)) {
-		nemo_window_go_back (window);
+		nemo_window_back_or_forward (window, TRUE, 0, 0);
 		handled = TRUE; 
 	} else if (mouse_extra_buttons && (event->button == mouse_forward_button)) {
-		nemo_window_go_forward (window);
+		nemo_window_back_or_forward (window, FALSE, 0, 0);
 		handled = TRUE;
 	} else if (GTK_WIDGET_CLASS (nemo_window_parent_class)->button_press_event) {
 		handled = GTK_WIDGET_CLASS (nemo_window_parent_class)->button_press_event (widget, event);
@@ -2005,6 +2062,8 @@ use_extra_mouse_buttons_changed (gpointer callback_data)
 static void
 nemo_window_init (NemoWindow *window)
 {
+    GtkWindowGroup *window_group;
+
 	window->details = G_TYPE_INSTANCE_GET_PRIVATE (window, NEMO_TYPE_WINDOW, NemoWindowDetails);
 
 	window->details->panes = NULL;
@@ -2018,6 +2077,18 @@ nemo_window_init (NemoWindow *window)
     window->details->show_sidebar = g_settings_get_boolean (nemo_window_state,
                                                             NEMO_WINDOW_STATE_START_WITH_SIDEBAR);
 
+    window->details->ignore_meta_view_id = NULL;
+    window->details->ignore_meta_zoom_level = -1;
+    window->details->ignore_meta_visible_columns = NULL;
+    window->details->ignore_meta_column_order = NULL;
+    window->details->ignore_meta_sort_column = NULL;
+    window->details->ignore_meta_sort_direction = SORT_NULL;
+    window->details->ignore_meta_tighter_layout = TIGHTER_NULL;
+
+	window_group = gtk_window_group_new ();
+	gtk_window_group_add_window (window_group, GTK_WINDOW (window));
+	g_object_unref (window_group);
+
 	/* Set initial window title */
 	gtk_window_set_title (GTK_WINDOW (window), _("Nemo"));
 }
@@ -2027,6 +2098,7 @@ real_get_icon (NemoWindow *window,
                NemoWindowSlot *slot)
 {
         return nemo_file_get_icon (slot->viewed_file, 48,
+                       gtk_widget_get_scale_factor (GTK_WIDGET (window)),
 				       NEMO_FILE_ICON_FLAGS_IGNORE_VISITING |
 				       NEMO_FILE_ICON_FLAGS_USE_MOUNT_ICON);
 }
@@ -2062,6 +2134,7 @@ nemo_window_class_init (NemoWindowClass *class)
     wclass->key_release_event = nemo_window_key_release_event;
 	wclass->window_state_event = nemo_window_state_event;
 	wclass->button_press_event = nemo_window_button_press_event;
+	wclass->delete_event = nemo_window_delete_event;
 
 	class->get_icon = real_get_icon;
 	class->close = real_window_close;
@@ -2089,13 +2162,13 @@ nemo_window_class_init (NemoWindowClass *class)
                               G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
 	signals[GO_UP] =
-		g_signal_new ("go_up",
+		g_signal_new ("go-up",
 			      G_TYPE_FROM_CLASS (class),
 			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 			      G_STRUCT_OFFSET (NemoWindowClass, go_up),
-			      g_signal_accumulator_true_handled, NULL,
+			      NULL, NULL,
 			      g_cclosure_marshal_generic,
-			      G_TYPE_BOOLEAN, 1, G_TYPE_BOOLEAN);
+			      G_TYPE_NONE, 0);
 	signals[RELOAD] =
 		g_signal_new ("reload",
 			      G_TYPE_FROM_CLASS (class),
@@ -2129,16 +2202,37 @@ nemo_window_class_init (NemoWindowClass *class)
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
+	signals[SLOT_ADDED] =
+		g_signal_new ("slot-added",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      0,
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, NEMO_TYPE_WINDOW_SLOT);
+	signals[SLOT_REMOVED] =
+		g_signal_new ("slot-removed",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      0,
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, NEMO_TYPE_WINDOW_SLOT);
 
 	binding_set = gtk_binding_set_by_class (class);
 	gtk_binding_entry_add_signal (binding_set, GDK_KEY_BackSpace, 0,
-				      "go_up", 1,
-				      G_TYPE_BOOLEAN, FALSE);
+				      "go-up", 0);
 	gtk_binding_entry_add_signal (binding_set, GDK_KEY_F5, 0,
 				      "reload", 0);
 	gtk_binding_entry_add_signal (binding_set, GDK_KEY_slash, 0,
 				      "prompt-for-location", 1,
 				      G_TYPE_STRING, "/");
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_KP_Divide, 0,
+				      "prompt-for-location", 1,
+				      G_TYPE_STRING, "/");
+	gtk_binding_entry_add_signal (binding_set, GDK_KEY_asciitilde, 0,
+				      "prompt-for-location", 1,
+				      G_TYPE_STRING, "~");
 
 	class->reload = nemo_window_reload;
 	class->go_up = nemo_window_go_up_signal;
@@ -2163,6 +2257,14 @@ nemo_window_class_init (NemoWindowClass *class)
 	g_type_class_add_private (oclass, sizeof (NemoWindowDetails));
 }
 
+NemoWindow *
+nemo_window_new (GdkScreen *screen)
+{
+	return g_object_new (NEMO_TYPE_WINDOW,
+			     "screen", screen,
+			     NULL);
+}
+
 void
 nemo_window_split_view_on (NemoWindow *window)
 {
@@ -2186,7 +2288,7 @@ nemo_window_split_view_on (NemoWindow *window)
 		location = g_file_new_for_path (g_get_home_dir ());
 	}
 
-	nemo_window_slot_go_to (slot, location, FALSE);
+	nemo_window_slot_open_location (slot, location, 0);
 	g_object_unref (location);
 
 	window_set_search_action_text (window, FALSE);
@@ -2195,7 +2297,6 @@ nemo_window_split_view_on (NemoWindow *window)
 void
 nemo_window_split_view_off (NemoWindow *window)
 {
-	gboolean show_label_search_icon_toolbar;
 	NemoWindowPane *pane, *active_pane;
 	GList *l, *next;
 
@@ -2215,9 +2316,6 @@ nemo_window_split_view_off (NemoWindow *window)
 					      active_pane->action_group);
 
 	nemo_window_update_show_hide_menu_items (window);
-
-	show_label_search_icon_toolbar = g_settings_get_boolean (nemo_preferences, NEMO_PREFERENCES_SHOW_LABEL_SEARCH_ICON_TOOLBAR);
-	window_set_search_action_text (window, show_label_search_icon_toolbar);
 }
 
 gboolean
@@ -2266,3 +2364,169 @@ nemo_window_get_show_sidebar (NemoWindow *window)
 {
     return window->details->show_sidebar;
 }
+
+const gchar *
+nemo_window_get_ignore_meta_view_id (NemoWindow *window)
+{
+    return window->details->ignore_meta_view_id;
+}
+
+void
+nemo_window_set_ignore_meta_view_id (NemoWindow *window, const gchar *id)
+{
+    if (id != NULL) {
+        gchar *old_id = window->details->ignore_meta_view_id;
+        if (g_strcmp0 (old_id, id) != 0) {
+            nemo_window_set_ignore_meta_zoom_level (window, -1);
+        }
+        window->details->ignore_meta_view_id = g_strdup (id);
+        g_free (old_id);
+    }
+}
+
+gint
+nemo_window_get_ignore_meta_zoom_level (NemoWindow *window)
+{
+    return window->details->ignore_meta_zoom_level;
+}
+
+void
+nemo_window_set_ignore_meta_zoom_level (NemoWindow *window, gint level)
+{
+    window->details->ignore_meta_zoom_level = level;
+}
+
+/* FIXME:
+ *
+ * Remove this and just use g_list_copy_deep 
+ * when we no longer need to support GLib < 2.34
+ *
+ */
+
+static GList *
+list_copy_deep (GList *list, GCopyFunc func, gpointer user_data)
+{
+  GList *new_list = NULL;
+
+  if (list)
+    {
+      GList *last;
+
+      new_list = g_slice_new (GList);
+      if (func)
+        new_list->data = func (list->data, user_data);
+      else
+        new_list->data = list->data;
+      new_list->prev = NULL;
+      last = new_list;
+      list = list->next;
+      while (list)
+    {
+      last->next = g_slice_new (GList);
+      last->next->prev = last;
+      last = last->next;
+      if (func)
+        last->data = func (list->data, user_data);
+      else
+        last->data = list->data;
+      list = list->next;
+    }
+      last->next = NULL;
+    }
+
+  return new_list;
+}
+
+GList *
+nemo_window_get_ignore_meta_visible_columns (NemoWindow *window)
+{
+    return list_copy_deep (window->details->ignore_meta_visible_columns, (GCopyFunc) g_strdup, NULL);
+}
+
+void
+nemo_window_set_ignore_meta_visible_columns (NemoWindow *window, GList *list)
+{
+    GList *old = window->details->ignore_meta_visible_columns;
+    window->details->ignore_meta_visible_columns = list != NULL ? list_copy_deep (list, (GCopyFunc) g_strdup, NULL) :
+                                                                  NULL;
+    if (old != NULL)
+        g_list_free_full (old, g_free);
+}
+
+GList *
+nemo_window_get_ignore_meta_column_order (NemoWindow *window)
+{
+    return list_copy_deep (window->details->ignore_meta_column_order, (GCopyFunc) g_strdup, NULL);
+}
+
+void
+nemo_window_set_ignore_meta_column_order (NemoWindow *window, GList *list)
+{
+    GList *old = window->details->ignore_meta_column_order;
+    window->details->ignore_meta_column_order = list != NULL ? list_copy_deep (list, (GCopyFunc) g_strdup, NULL) :
+                                                               NULL;
+    if (old != NULL)
+        g_list_free_full (old, g_free);
+}
+
+const gchar *
+nemo_window_get_ignore_meta_sort_column (NemoWindow *window)
+{
+    return window->details->ignore_meta_sort_column;
+}
+
+void
+nemo_window_set_ignore_meta_sort_column (NemoWindow *window, const gchar *column)
+{
+    if (column != NULL) {
+        gchar *old_column = window->details->ignore_meta_sort_column;
+        window->details->ignore_meta_sort_column = g_strdup (column);
+        g_free (old_column);
+    }
+}
+
+gint
+nemo_window_get_ignore_meta_sort_direction (NemoWindow *window)
+{
+    return window->details->ignore_meta_sort_direction;
+}
+
+void
+nemo_window_set_ignore_meta_sort_direction (NemoWindow *window, gint direction)
+{
+    window->details->ignore_meta_sort_direction = direction;
+}
+
+gint
+nemo_window_get_ignore_meta_tighter_layout (NemoWindow *window)
+{
+    return window->details->ignore_meta_tighter_layout;
+}
+
+void
+nemo_window_set_ignore_meta_tighter_layout (NemoWindow *window, gint tighter)
+{
+    window->details->ignore_meta_tighter_layout = tighter;
+}
+
+NemoWindowOpenFlags
+nemo_event_get_window_open_flags (void)
+{
+	NemoWindowOpenFlags flags = 0;
+	GdkEvent *event;
+
+	event = gtk_get_current_event ();
+
+	if (event == NULL) {
+		return flags;
+	}
+
+	if ((event->type == GDK_BUTTON_PRESS || event->type == GDK_BUTTON_RELEASE) &&
+	    (event->button.button == 2)) {
+		flags |= NEMO_WINDOW_OPEN_FLAG_NEW_TAB;
+	}
+
+	gdk_event_free (event);
+
+	return flags;
+}
