@@ -66,6 +66,18 @@ typedef enum {
 	NEMO_WINDOW_OPEN_SLOT_APPEND = 1
 }  NemoWindowOpenSlotFlags;
 
+enum {
+    SORT_NULL = -1,
+    SORT_ASCENDING = 0,
+    SORT_DESCENDING = 1
+};
+
+enum {
+    TIGHTER_NULL = -1,
+    TIGHTER_NO = 0,
+    TIGHTER_YES = 1
+};
+
 #define NEMO_WINDOW_SIDEBAR_PLACES "places"
 #define NEMO_WINDOW_SIDEBAR_TREE "tree"
 
@@ -85,9 +97,8 @@ typedef struct {
         void   (* close) (NemoWindow *window);
 
         /* Signals used only for keybindings */
-        gboolean (* go_up)  (NemoWindow *window,
-                             gboolean close);
-	void     (* reload) (NemoWindow *window);
+        void   (* go_up)  (NemoWindow *window);
+	void   (* reload) (NemoWindow *window);
 } NemoWindowClass;
 
 struct NemoWindow {
@@ -97,6 +108,7 @@ struct NemoWindow {
 };
 
 GType            nemo_window_get_type             (void);
+NemoWindow * nemo_window_new                  (GdkScreen         *screen);
 void             nemo_window_close                (NemoWindow    *window);
 
 void             nemo_window_connect_content_view (NemoWindow    *window,
@@ -130,15 +142,21 @@ void                 nemo_window_report_load_underway  (NemoWindow *window,
 void                 nemo_window_view_visible          (NemoWindow *window,
                                                             NemoView *view);
 NemoWindowSlot * nemo_window_get_active_slot       (NemoWindow *window);
+GList *              nemo_window_get_panes             (NemoWindow *window);
+NemoWindowSlot * nemo_window_get_active_slot       (NemoWindow *window);
 void                 nemo_window_push_status           (NemoWindow *window,
                                                             const char *text);
+GtkWidget *          nemo_window_ensure_location_bar   (NemoWindow *window);
+void                 nemo_window_sync_location_widgets (NemoWindow *window);
+void                 nemo_window_sync_search_widgets   (NemoWindow *window);
+void                 nemo_window_grab_focus            (NemoWindow *window);
 
 void     nemo_window_hide_sidebar         (NemoWindow *window);
 void     nemo_window_show_sidebar         (NemoWindow *window);
 void     nemo_window_back_or_forward      (NemoWindow *window,
                                                gboolean        back,
                                                guint           distance,
-                                               gboolean        new_tab);
+                                               NemoWindowOpenFlags flags);
 void     nemo_window_split_view_on        (NemoWindow *window);
 void     nemo_window_split_view_off       (NemoWindow *window);
 gboolean nemo_window_split_view_showing   (NemoWindow *window);
@@ -157,4 +175,24 @@ void    nemo_window_set_show_sidebar (NemoWindow *window,
 
 gboolean  nemo_window_get_show_sidebar (NemoWindow *window);
 
+const gchar *nemo_window_get_ignore_meta_view_id (NemoWindow *window);
+void         nemo_window_set_ignore_meta_view_id (NemoWindow *window, const gchar *id);
+gint         nemo_window_get_ignore_meta_zoom_level (NemoWindow *window);
+void         nemo_window_set_ignore_meta_zoom_level (NemoWindow *window, gint level);
+GList       *nemo_window_get_ignore_meta_visible_columns (NemoWindow *window);
+void         nemo_window_set_ignore_meta_visible_columns (NemoWindow *window, GList *list);
+GList       *nemo_window_get_ignore_meta_column_order (NemoWindow *window);
+void         nemo_window_set_ignore_meta_column_order (NemoWindow *window, GList *list);
+const gchar *nemo_window_get_ignore_meta_sort_column (NemoWindow *window);
+void         nemo_window_set_ignore_meta_sort_column (NemoWindow *window, const gchar *column);
+gint         nemo_window_get_ignore_meta_sort_direction (NemoWindow *window);
+void         nemo_window_set_ignore_meta_sort_direction (NemoWindow *window, gint direction);
+gint         nemo_window_get_ignore_meta_tighter_layout (NemoWindow *window);
+void         nemo_window_set_ignore_meta_tighter_layout (NemoWindow *window, gint tighter);
+
+NemoWindowOpenFlags nemo_event_get_window_open_flags   (void);
+
+void nemo_window_slot_added (NemoWindow *window,  NemoWindowSlot *slot);
+void nemo_window_slot_removed (NemoWindow *window,  NemoWindowSlot *slot);
+
 #endif
